@@ -6,7 +6,7 @@
 
   Author:  Illiya Yalovoy <yalovoy@gmail.com>
 
-  $Id: DiscoveryDruid.cpp,v 1.25 2006/07/24 00:10:02 vkurland Exp $
+  $Id: DiscoveryDruid.cpp,v 1.26 2006/09/08 03:01:46 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -1830,8 +1830,12 @@ void DiscoveryDruid::createRealObjects()
 
                 if (od.interfaces.size()==0) 
                 {
-                    Interface *itf= Interface::cast(om->createObject(o,Interface::TYPENAME,"nic1"));
-                    IPv4 *ipv4= IPv4::cast(om->createObject(itf,IPv4::TYPENAME,a.c_str()));
+                    Interface *itf= Interface::cast(
+                        om->createObject(o,Interface::TYPENAME,"nic1")
+                    );
+                    IPv4 *ipv4= IPv4::cast(
+                        om->createObject(itf,IPv4::TYPENAME,a.c_str())
+                    );
                     
                     
                     ipv4->setAddress(a);
@@ -1877,7 +1881,9 @@ void DiscoveryDruid::createRealObjects()
                 }
             }else if (type==Network::TYPENAME)
             {
-                Network *net=dynamic_cast<Network*>( om->createObject(type,name) );
+                Network *net=dynamic_cast<Network*>(
+                    om->createObject(type.c_str(),name.c_str())
+                );
                 assert(net!=NULL);
                 net->setName(name);
                 net->setAddress(IPAddress(a));
@@ -1885,7 +1891,9 @@ void DiscoveryDruid::createRealObjects()
                 om->moveObject(libs->currentText(), net);
             }else if (type==IPv4::TYPENAME)
             {
-                IPv4 *obj=dynamic_cast<IPv4*>( om->createObject(type.c_str(),name.c_str()) );
+                IPv4 *obj=dynamic_cast<IPv4*>(
+                    om->createObject(type.c_str(),name.c_str())
+                );
                 assert(obj!=NULL);
                 obj->setName(name);
                 obj->setAddress(IPAddress(a));
