@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: FWWindow.cpp,v 1.200 2006/07/19 06:47:51 vkurland Exp $
+  $Id: FWWindow.cpp,v 1.208 2006/09/16 17:11:41 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -48,6 +48,7 @@
 #include "ObjectTreeView.h"
 #include "DiscoveryDruid.h"
 #include "FindObjectWidget.h"
+#include "FindWhereUsedWidget.h"
 #include "longTextDialog.h"
 
 #include "rcsfilesavedialog_q.h"
@@ -140,6 +141,7 @@
 #include <qprinter.h>
 #include <qwidgetstack.h>
 #include <qlistbox.h>
+#include <qeventloop.h>
 
 using namespace libfwbuilder;
 using namespace std;
@@ -156,14 +158,24 @@ FWWindow::FWWindow() : FWBMainWindow_q()
     lastFirewallIdx = -2;
     delete treeFrame;
     
+    instd = NULL;
 
     changingTabs = false;
+    ruleSetRedrawPending = false;
+
+
+    setSafeMode(false);
+    setStartupFileName("");
 
     printer = new QPrinter(QPrinter::HighResolution);
 
     noFirewalls = tr("No firewalls defined");
 
     autosaveTimer = new QTimer(this);
+    instDialogOnScreenTimer = new QTimer(this);
+
+    connect(instDialogOnScreenTimer, SIGNAL(timeout()), this, SLOT(killInstDialog()));
+    instDialogOnScreenTimer->start(1000);
 
 // om is a global var
     om=new ObjectManipulator( objInfoSplitter );
@@ -211,6 +223,11 @@ FWWindow::FWWindow() : FWBMainWindow_q()
     auxiliaryPanelLayout->addWidget( findObjectWidget );
     connect( findObjectWidget, SIGNAL( close() ), this, SLOT( closeAuxiliaryPanel() ) );
     
+    findWhereUsedWidget = new FindWhereUsedWidget(auxiliaryPanel, "findWhereUsedWidget");
+    findWhereUsedWidget->setFocusPolicy( FindObjectWidget::NoFocus );
+    auxiliaryPanelLayout->addWidget( findWhereUsedWidget );
+    findWhereUsedWidget->hide();
+    connect( findWhereUsedWidget, SIGNAL( close() ), this, SLOT( closeAuxiliaryPanel() ) );
     auxiliaryPanel->hide();
     om->show();
 
@@ -222,6 +239,76 @@ FWWindow::~FWWindow()
     if (rcs!=NULL) delete rcs;
 }
 
+void FWWindow::killInstDialog()
+{
+    if (instd!=NULL && !instd->isVisible())
+    {
+        if (fwbdebug) qDebug("killing instDialog...");
+        delete instd;
+        instd = NULL;
+    }
+}
+
+void FWWindow::startupLoad()
+{
+    if (fwbdebug) qDebug("startup: load everything ...");
+
+    int sa = st->getStartupAction();
+
+    if (safeMode)  mw->load(NULL);
+    else
+    {
+        if (startupFileName.isEmpty() && sa==1) // load last edited
+        {
+            startupFileName = st->getLastEdited();
+        }
+
+        if ( !startupFileName.isEmpty() )
+        {
+            try
+            {
+                RCS *rcs=new RCS(startupFileName);
+                rcs->co();
+                load(NULL,rcs);
+            } catch (FWException &ex)
+            {
+                qDebug("Exception: %s",ex.toString().c_str());
+                load(NULL);
+            }
+        } else
+        {
+            load(NULL); // load standard objects
+        }
+    }
+
+    QString id = st->getStr("UI/visibleFirewall");
+    FWObject *show_fw=NULL;
+    if ( !id.isEmpty() ) show_fw = mw->db()->getById(id.latin1(),true);
+
+    id = st->getStr("UI/visibleObject");
+    FWObject *show_obj=NULL;
+    if ( !id.isEmpty() ) show_obj = mw->db()->getById(id.latin1(),true);
+
+    showFirewalls( show_fw==NULL );
+
+    if ( sa==1 && !safeMode )
+    {
+        if (show_fw)
+        {
+            if (fwbdebug)
+                qDebug("open firewall %s",show_fw->getName().c_str());
+            showFirewall( show_fw );
+        }
+
+        if (show_obj)
+        {
+            if (fwbdebug)
+                qDebug("open object %s",show_obj->getName().c_str());
+            om->openObject( show_obj );
+        }
+    }
+}
+
 void FWWindow::clearFirewallTabs()
 {
     if (fwbdebug) qDebug("FWWindow::clearFirewallTabs");
@@ -389,7 +476,7 @@ void FWWindow::fileNew()
         else  load(this);
 
         visibleFirewall = NULL;
-        showFirewalls();
+        showFirewalls( false );
 
 
         setFileName(nfn);
@@ -437,7 +524,7 @@ void FWWindow::fileOpen()
 /***********************************************************************/
 
         load(this, rcs );
-        showFirewalls();
+        showFirewalls( true );
 
         if (rcs->isTemp()) unlink(rcs->getFileName().latin1());
     }
@@ -471,7 +558,7 @@ void FWWindow::fileClose()
 
     if (fwbdebug) qDebug("FWWindow::fileClose(): show firewalls");
 
-    showFirewalls();
+    showFirewalls( false );
 
     if (fwbdebug) qDebug("FWWindow::fileClose(): all done");
 
@@ -482,8 +569,10 @@ void FWWindow::fileSave()
 {
     QStatusBar *sb = statusBar();
     sb->message( tr("Saving data to file...") );
+    QApplication::eventLoop()->processEvents(QEventLoop::ExcludeUserInput,100);
     save();
     sb->clear();
+    QApplication::eventLoop()->processEvents(QEventLoop::ExcludeUserInput,100);
 }
 
 void FWWindow::fileSaveAs()
@@ -558,7 +647,7 @@ void FWWindow::fileCommit()
         load(this);
         return;
     }
-    showFirewalls();
+    showFirewalls( true );
 }
 
 /*
@@ -600,7 +689,7 @@ void FWWindow::fileDiscard()
         }
 /***********************************************************************/
 
-        showFirewalls();
+        showFirewalls( true );
     }
 }
 
@@ -652,9 +741,11 @@ void FWWindow::toolsDiscoveryDruid()
    DiscoveryDruid druid(this); 
    druid.exec();
 }
+
 void FWWindow::load(QWidget *dialogs_parent)
 {
     if (fwbdebug) qDebug("FWWindow::load(): start");
+    QStatusBar *sb = statusBar();
 
     editingStandardLib = false;
     editingTemplateLib = false;
@@ -683,6 +774,9 @@ void FWWindow::load(QWidget *dialogs_parent)
 
         if (fwbdebug) qDebug("FWWindow::load(): loading objects");
 
+        sb->message( tr("Loading system objects...") );
+        QApplication::eventLoop()->processEvents(QEventLoop::ExcludeUserInput,100);
+
 // always loading system objects
         objdb->load( sysfname, &upgrade_predicate, librespath);
         objdb->setFileName("");
@@ -755,6 +849,8 @@ void FWWindow::load(QWidget *dialogs_parent)
 
 void FWWindow::load(QWidget *dialogs_parent,RCS *_rcs)
 {
+    QStatusBar *sb = statusBar();
+
     fd->reset();
 
     editingStandardLib = false;
@@ -788,14 +884,24 @@ void FWWindow::load(QWidget *dialogs_parent,RCS *_rcs)
         objdb->setReadOnly( false );
 
 // always loading system objects
+        sb->message( tr("Loading system objects...") );
+        QApplication::eventLoop()->processEvents(QEventLoop::ExcludeUserInput,100);
+
         objdb->load( sysfname, &upgrade_predicate, librespath);
         objdb->setFileName("");
 
-/* objects from a data file are in database ndb */
+// objects from a data file are in database ndb
+
+        sb->message( tr("Reading and parsing data file...") );
+        QApplication::eventLoop()->processEvents(QEventLoop::ExcludeUserInput,100);
+
         FWObjectDatabase *ndb = new FWObjectDatabase();
         ndb->load(rcs->getFileName().latin1(), &upgrade_predicate,librespath);
         time_t   oldtimestamp = ndb->getTimeLastModified();
 
+        sb->clear();
+        QApplication::eventLoop()->processEvents(QEventLoop::ExcludeUserInput,100);
+
 /* loadingLib is true if user wants to open a library or master library file */
         bool loadingLib         = editingLibrary();
 
@@ -856,6 +962,9 @@ void FWWindow::load(QWidget *dialogs_parent,RCS *_rcs)
             }
         }
 
+        sb->message( tr("Merging with system objects...") );
+        QApplication::eventLoop()->processEvents(QEventLoop::ExcludeUserInput,100);
+
         MergeConflictRes mcr(dlgp);
         objdb->merge(ndb, &mcr);
 
@@ -865,6 +974,9 @@ void FWWindow::load(QWidget *dialogs_parent,RCS *_rcs)
         objdb->resetTimeLastModified(oldtimestamp);
         objdb->setDirty(false);
 
+        sb->clear();
+        QApplication::eventLoop()->processEvents(QEventLoop::ExcludeUserInput,100);
+
 /*
  * TODO: we should create new FWObjectDatabase object and assign db
  * instead of using singleton
@@ -1031,7 +1143,18 @@ void FWWindow::load(QWidget *dialogs_parent,RCS *_rcs)
 // clear dirty flag for all objects, recursively
     if (!forceSave)  db()->setDirty(false);
 
+    sb->message( tr("Building object tree...") );
+    QApplication::eventLoop()->processEvents(QEventLoop::ExcludeUserInput,100);
     om->loadObjects();
+    QApplication::eventLoop()->processEvents(QEventLoop::ExcludeUserInput,1000);
+
+    sb->message( tr("Indexing...") );
+    QApplication::eventLoop()->processEvents(QEventLoop::ExcludeUserInput,100);
+    db()->reIndex();
+    
+
+    sb->clear();
+    QApplication::eventLoop()->processEvents(QEventLoop::ExcludeUserInput,100);
 
     setupAutoSave();
 }
@@ -1254,7 +1377,7 @@ void FWWindow::fileImport()
     loadLibrary( fname.latin1() );
 
     om->loadObjects();
-    showFirewalls();
+    showFirewalls( true );
 
 //    addOnLibs->add( fname.latin1() );
 }
@@ -1515,7 +1638,7 @@ void FWWindow::fileExport()
     exportLibraryTo(fname,selectedLibs,ed.exportRO->isChecked());
 }
 
-void FWWindow::showFirewalls()
+void FWWindow::showFirewalls(bool open_first_firewall)
 {
     if (fwbdebug)  qDebug("FWWindow::showFirewalls");
 
@@ -1552,8 +1675,11 @@ void FWWindow::showFirewalls()
     for (list<FWObject*>::iterator m=fl.begin(); m!=fl.end(); m++)
         addFirewallToList( *m );
 
-    fwList->setCurrentItem( 0 );
-    openFirewall( 0 );
+    if (open_first_firewall)
+    {
+        fwList->setCurrentItem( 0 );
+        openFirewall( 0 );
+    }
 }
 
 int  FWWindow::findFirewallInList(FWObject *f)
@@ -1610,24 +1736,6 @@ void FWWindow::removeFirewallFromList(FWObject *o)
     }
 }
 
-void FWWindow::showFirewall(FWObject *obj)
-{
-    if (firewalls.size()>0)
-    {
-        vector<FWObject*>::iterator i;
-        int n=0;
-        for (i=firewalls.begin(); i!=firewalls.end(); i++,n++)
-        {
-            if ( (*i)->getId()==obj->getId() )
-            {
-                fwList->setCurrentItem( n );
-                openFirewall( n );
-                return;
-            }
-        }
-    }
-}
-
 void FWWindow::ensureObjectVisibleInRules(FWReference *obj)
 {
     FWObject *p=obj;
@@ -1711,14 +1819,20 @@ void FWWindow::setPolicyBranchTabName(PolicyRule *rule)
 
 void FWWindow::addPolicyBranchTab(PolicyRule *rule)
 {
+    FWOptions *ropt = rule->getOptionsObject();
+    QString branchName = ropt->getStr("branch_name").c_str();
+
+    QStatusBar *sb = statusBar();
+    sb->message( tr("Building branch policy view '%1'...").arg(branchName) );
+    QApplication::eventLoop()->processEvents(QEventLoop::ExcludeUserInput,1000);
+    if (fwbdebug)  qDebug("FWWindow::reopenFirewall()   adding branch tab");
+
     Policy *subset = Policy::cast(rule->getBranch());
     if (subset==NULL)
     {
         subset = new Policy();
         rule->add(subset);
     }
-    FWOptions *ropt = rule->getOptionsObject();
-    QString branchName = ropt->getStr("branch_name").c_str();
     if (ruleSetViews.count(subset)==0)
     {
         RuleSetView *rsv;
@@ -1739,10 +1853,24 @@ void FWWindow::removePolicyBranchTab(PolicyRule *rule)
     ruleSetViews.erase(subset);
 }
 
+void FWWindow::scheduleRuleSetRedraw()
+{
+    ruleSetRedrawPending = true;
+    QTimer::singleShot( 0, this, SLOT(redrawRuleSets()) );
+}
+
+void FWWindow::redrawRuleSets()
+{
+    ruleSetRedrawPending = false;
+    reopenFirewall();
+}
+
 void FWWindow::reopenFirewall()
 {
     if (fwbdebug)  qDebug("FWWindow::reopenFirewall()");
 
+    if (ruleSetRedrawPending) return;
+
     int currentPage = ruleSets->currentPageIndex();
 
     changingTabs=true;
@@ -1756,11 +1884,19 @@ void FWWindow::reopenFirewall()
         return;
     }
 
+    QStatusBar *sb = statusBar();
+    sb->message( tr("Building policy view...") );
+    QApplication::eventLoop()->processEvents(QEventLoop::ExcludeUserInput,1000);
+    if (fwbdebug)  qDebug("FWWindow::reopenFirewall()   adding Policy tab");
+
     RuleSetView *rsv;
     Policy *pol=Policy::cast(visibleFirewall->getFirstByType(Policy::TYPENAME));
     ruleSets->addTab( rsv=new PolicyView(pol,NULL) , tr("Policy") );
     ruleSetViews[pol]=rsv;
 
+// let the GUI process events to display new tab
+    QApplication::eventLoop()->processEvents(QEventLoop::ExcludeUserInput,1000);
+
 // as of 2.1.5 we have rule branches :-)
 // so far branches are only supported in policy rules because only there
 // we have action which we use to define branching rules
@@ -1772,18 +1908,36 @@ void FWWindow::reopenFirewall()
             addPolicyBranchTab(rule);
     }
 
+// let the GUI process events to display new tab(s)
+    QApplication::eventLoop()->processEvents(QEventLoop::ExcludeUserInput,1000);
+
+    sb->message( tr("Building NAT view...") );
+    QApplication::eventLoop()->processEvents(QEventLoop::ExcludeUserInput,100);
+    if (fwbdebug)  qDebug("FWWindow::reopenFirewall()   adding NAT tab");
+
     NAT *nat  = NAT::cast(visibleFirewall->getFirstByType(NAT::TYPENAME));
     ruleSets->addTab( rsv=new NATView(nat,NULL) , tr("NAT") );
     ruleSetViews[nat]=rsv;
 
+// let the GUI process events to display new tab
+    QApplication::eventLoop()->processEvents(QEventLoop::ExcludeUserInput,1000);
+
     if (Resources::getTargetCapabilityBool(visibleFirewall->getStr("host_OS"),
                                            "supports_routing"))
     {
+        sb->message( tr("Building routing view...") );
+        QApplication::eventLoop()->processEvents(QEventLoop::ExcludeUserInput,100);
+        if (fwbdebug)  qDebug("FWWindow::reopenFirewall()   adding Routing tab");
+
         Routing *r = Routing::cast(visibleFirewall->getFirstByType(Routing::TYPENAME));
         ruleSets->addTab( rsv=new RoutingView(r,NULL) , tr("Routing") );
         ruleSetViews[r]=rsv;
     }
 
+    sb->clear();
+    QApplication::eventLoop()->processEvents(QEventLoop::ExcludeUserInput,100);
+    if (fwbdebug)  qDebug("FWWindow::reopenFirewall()   all tabs are done");
+
     ruleSets->setCurrentPage( currentPage );
 
     changingTabs=false;
@@ -1805,6 +1959,24 @@ void FWWindow::reopenFirewall()
     ruleSets->show();
 }
 
+void FWWindow::showFirewall(FWObject *obj)
+{
+    if (firewalls.size()>0)
+    {
+        vector<FWObject*>::iterator i;
+        int n=0;
+        for (i=firewalls.begin(); i!=firewalls.end(); i++,n++)
+        {
+            if ( (*i)->getId()==obj->getId() )
+            {
+                fwList->setCurrentItem( n );
+                openFirewall( n );
+                return;
+            }
+        }
+    }
+}
+
 void FWWindow::openFirewall( int idx )
 {
     if (fwbdebug)
@@ -1816,11 +1988,9 @@ void FWWindow::openFirewall( int idx )
             requestEditorOwnership(NULL,NULL,ObjectEditor::optNone,true))
         {
             oe->blank();
-            FWObject *fw=firewalls[idx];
+            FWObject *fw = firewalls[idx];
+            showFirewallRuleSets(fw);
             visibleFirewall = fw;
-            findObjectWidget->firewallOpened(Firewall::cast(fw));
-            firewallName->setText(QString::fromUtf8(fw->getName().c_str()));
-            reopenFirewall();
             om->openObject(fw);
             lastFirewallIdx=idx;
         } else
@@ -1829,6 +1999,20 @@ void FWWindow::openFirewall( int idx )
         visibleFirewall = NULL;
 }
 
+void FWWindow::showFirewallRuleSets( FWObject *fw )
+{
+    if (fwbdebug)
+        qDebug("FWWindow::showFirewallRuleSets");
+
+    if (fw==NULL) return;
+
+    visibleFirewall = fw;
+    findObjectWidget->firewallOpened(Firewall::cast(fw));
+    firewallName->setText(QString::fromUtf8(fw->getName().c_str()));
+    scheduleRuleSetRedraw();
+    //reopenFirewall();
+}
+
 void FWWindow::selectRules()
 {
 //    om->unselect();
@@ -1949,10 +2133,11 @@ void FWWindow::compile()
 
     fileSave();
 
-    instDialog *id=new instDialog(this,BATCH_COMPILE);
- 
-    id->exec();
-    delete id;
+    instd = new instDialog(NULL,BATCH_COMPILE);
+    instd->show();
+
+//    id->exec();
+//    delete id;
 }
 
 void FWWindow::compile(set<Firewall*> vf)
@@ -1965,7 +2150,7 @@ void FWWindow::compile(set<Firewall*> vf)
 
     fileSave();
 
-    instDialog *id=new instDialog(this,BATCH_COMPILE);
+    instDialog *id = new instDialog(NULL,BATCH_COMPILE);
  
     id->clearReqFirewalls();
     
@@ -1974,14 +2159,16 @@ void FWWindow::compile(set<Firewall*> vf)
     {
         id->addReqFirewall(*i);
     }
-    
-    id->exec();
-    delete id;
+    instd = id;
+    instd->show();
+
+//    id->exec();
+//    delete id;
 }
 
 void FWWindow::install(set<Firewall*> vf)
 {
-    instDialog *id=new instDialog(this,BATCH_INSTALL);
+    instDialog *id=new instDialog(NULL,BATCH_INSTALL);
  
     id->clearReqFirewalls();
     
@@ -1990,17 +2177,21 @@ void FWWindow::install(set<Firewall*> vf)
     {
         id->addReqFirewall(*i);
     }
-    id->exec();
-    delete id;
-    
+
+    instd = id;
+    instd->show();
+
+//    id->exec();
+//    delete id;
 }
+
 void FWWindow::install()
 {
-    instDialog *id=new instDialog(this,BATCH_INSTALL);
- 
-    id->show();
-    id->exec();
-    delete id;
+    instd = new instDialog(NULL,BATCH_INSTALL);
+    instd->show();
+
+//    id->exec();
+//    delete id;
 }
 
 /*
@@ -2097,11 +2288,17 @@ void FWWindow::pasteRuleBelow()
     dynamic_cast<RuleSetView*>(ruleSets->currentPage())->pasteRuleBelow();
 }
 
-
-
 void FWWindow::search()
 {
+    findWhereUsedWidget->hide();
     auxiliaryPanel->show();
+    findObjectWidget->show();
+}
+void FWWindow::findWhereUsed(FWObject * obj)
+{
+    findObjectWidget->hide();
+    auxiliaryPanel->show();
+    findWhereUsedWidget->find(obj);
 }
 
 void FWWindow::showEvent( QShowEvent *ev)
@@ -2178,6 +2375,7 @@ RCS * FWWindow::getRCS()
 
 void FWWindow::findObject(FWObject *o)
 {
+    findWhereUsedWidget->hide();
     if (fwbdebug) qDebug("FWWindow::findObject");
     findObjectWidget->findObject(o);
     auxiliaryPanel->show();
@@ -2186,6 +2384,7 @@ void FWWindow::findObject(FWObject *o)
 
 void FWWindow::closeAuxiliaryPanel()
 {
+
     auxiliaryPanel->hide();
 }
 
