@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: FWWindowPrint.cpp,v 1.19.4.1 2005/09/04 20:49:14 vkurland Exp $
+  $Id: FWWindowPrint.cpp,v 1.22 2006/02/20 17:05:41 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -70,7 +70,6 @@
 #include "fwbuilder/FWReference.h"
 #include "fwbuilder/Interface.h"
 #include "fwbuilder/RuleSet.h"
-#include "fwbuilder/InterfacePolicy.h"
 
 #include "fwbuilder/CustomService.h"
 #include "fwbuilder/IPService.h"
@@ -161,7 +160,7 @@ list<FWObject*> findAllUsedByType(list<FWObject*> &result,FWObject *obj,const st
         FWObjectTypedChildIterator j=obj->findByType(Interface::TYPENAME);
         for ( ; j!=j.end(); ++j )
         {
-            ruleSet = (*j)->getFirstByType(InterfacePolicy::TYPENAME);
+          if ((ruleSet = (*j)->getFirstByType(InterfacePolicy::TYPENAME))!=NULL)
             findAllUsedByType(result,ruleSet,typeName);
         }
 
@@ -268,8 +267,7 @@ void printFirewall(FWObject *fw,
 
     QString platform = fw->getStr("platform").c_str();
     QString version  = fw->getStr("version").c_str();
-    QMap<QString,QString> vl = getVersionsForPlatform(platform);
-    QString readableVersion = vl[version];
+    QString readableVersion = getVersionString(platform,version);
     QString hostOS = fw->getStr("host_OS").c_str();
 
     pr.beginPage();   // resets yPos
@@ -314,37 +312,39 @@ void printFirewall(FWObject *fw,
     {
         Interface       *intf = Interface::cast(*j);
         InterfacePolicy *ip   = InterfacePolicy::cast((*j)->getFirstByType(InterfacePolicy::TYPENAME));
-
-        QString tabName;
-        if ( !intf->getLabel().empty() )
+        if (ip)
+        {
+          QString tabName;
+          if ( !intf->getLabel().empty() )
             tabName=QString::fromUtf8(intf->getLabel().c_str());
-        else
+          else
             tabName=QString::fromUtf8(intf->getName().c_str());
 
 //                ppd->genericProgressIndicator(ppdCounter++,QObject::tr("Processing policy for interface %1").arg(tabName));
-        if (fwbdebug)  qDebug("Interface policy for %s",tabName.latin1());
+          if (fwbdebug)  qDebug("Interface policy for %s",tabName.latin1());
 
 
-        if (newPageForSection)
-        {
-            pr.flushPage();
-            pr.beginPage();   // resets yPos
-        } else
+          if (newPageForSection)
+            {
+              pr.flushPage();
+              pr.beginPage();   // resets yPos
+            } else
             pr.printText(" ");
 
-        pr.printText(QObject::tr("Interface %1").arg(tabName));
+          pr.printText(QObject::tr("Interface %1").arg(tabName));
 
-        ruleView=new InterfacePolicyView(ip,NULL);
-        ruleView->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7,
-                                              (QSizePolicy::SizeType)7) );
-        ruleView->setHScrollBarMode(QScrollView::AlwaysOff);
-        ruleView->setVScrollBarMode(QScrollView::AlwaysOff);
+          ruleView=new InterfacePolicyView(ip,NULL);
+          ruleView->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7,
+                                                (QSizePolicy::SizeType)7) );
+          ruleView->setHScrollBarMode(QScrollView::AlwaysOff);
+          ruleView->setVScrollBarMode(QScrollView::AlwaysOff);
 
-        if (fwbdebug) qDebug("%dx%d",ruleView->width(),ruleView->height());
+          if (fwbdebug) qDebug("%dx%d",ruleView->width(),ruleView->height());
 
-        pr.printQTable(ruleView);
-//        pr.printPixmap(QPixmap::grabWidget(ruleView,0,0));
-        delete ruleView;
+          pr.printQTable(ruleView);
+          //        pr.printPixmap(QPixmap::grabWidget(ruleView,0,0));
+          delete ruleView;
+        }
     }
 
 //            ppd->genericProgressIndicator(ppdCounter++,QObject::tr("Processing NAT rules"));
