@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: FirewallDialog.cpp,v 1.39 2006/06/14 07:03:15 vkurland Exp $
+  $Id: FirewallDialog.cpp,v 1.40 2006/09/11 00:41:09 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -264,7 +264,8 @@ void FirewallDialog::applyChanges()
         if (fwbdebug)
             qDebug("FirewallDialog::applyChanges() scheduling call to reopenFirewall()");
         //mw->reopenFirewall();
-        QTimer::singleShot( 0, mw, SLOT(reopenFirewall()) );
+        //QTimer::singleShot( 0, mw, SLOT(reopenFirewall()) );
+        mw->scheduleRuleSetRedraw();
     }
 
     init=true;
