@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: GroupObjectDialog.cpp,v 1.52 2006/06/12 06:55:05 vkurland Exp $
+  $Id: GroupObjectDialog.cpp,v 1.53 2006/09/15 05:48:21 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -712,12 +712,12 @@ void GroupObjectDialog::cutObj()
 
 void GroupObjectDialog::pasteObj()
 {
-    vector<FWObject*>::iterator i;
+    vector<string>::iterator i;
     
     for (i= FWObjectClipboard::obj_clipboard->begin();
          i!=FWObjectClipboard::obj_clipboard->end(); ++i)
     {
-        insertObject( *i );
+        insertObject( mw->db()->findInIndex(*i) );
     }
 
 //    if (FWObjectClipboard::obj_clipboard->getObject()==NULL) return;
