@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: LibraryDialog.cpp,v 1.26 2004/12/18 03:03:31 vkurland Exp $
+  $Id: LibraryDialog.cpp,v 1.28 2006/05/13 06:53:05 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -75,7 +75,7 @@ void LibraryDialog::loadFWObject(FWObject *o)
 
     fillColor();
 
-    apply->setEnabled( false );
+    //apply->setEnabled( false );
 
     obj_name->setEnabled(!o->isReadOnly());
     setDisabledPalette(obj_name);
@@ -92,13 +92,14 @@ void LibraryDialog::loadFWObject(FWObject *o)
     
 void LibraryDialog::changed()
 {
-    apply->setEnabled( true );
+    //apply->setEnabled( true );
+    emit changed_sign();
 }
 
 void LibraryDialog::changeIds(FWObject *root)
 {
     if (FWBTree::isStandardId(root))
-        root->setId(FWObjectDatabase::db->generateUniqueId());
+        root->setId(FWObjectDatabase::generateUniqueId());
         
     for (FWObject::iterator i=root->begin(); i!=root->end(); i++)
         changeIds( *i );
@@ -125,7 +126,7 @@ void LibraryDialog::applyChanges()
     om->updateLibName(obj);
     if (color!=oldcolor) om->updateLibColor(obj);
 
-    apply->setEnabled( false );
+    //apply->setEnabled( false );
 }
 
 void LibraryDialog::discardChanges()
@@ -140,7 +141,7 @@ void LibraryDialog::validate(bool *res)
 
 void LibraryDialog::isChanged(bool *res)
 {
-    *res=(!init && apply->isEnabled());
+    //*res=(!init && apply->isEnabled());
 }
 
 void LibraryDialog::changeColor()
