@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: ObjectIconView.cpp,v 1.8 2004/06/30 05:43:35 vkurland Exp $
+  $Id: ObjectIconView.cpp,v 1.9 2005/09/07 16:40:50 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -27,6 +27,8 @@
 #include "config.h"
 #include "global.h"
 
+#include "fwbuilder/FWObjectDatabase.h"
+#include "FWWindow.h"
 #include "ObjectIconView.h"
 #include "ObjectIconViewItem.h"
 #include "FWObjectDrag.h"
