@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: ObjectManipulator.cpp,v 1.143 2006/07/23 15:48:02 vkurland Exp $
+  $Id: ObjectManipulator.cpp,v 1.151 2006/09/15 05:48:21 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -68,11 +68,13 @@
 #include <qlineedit.h>
 #include <qcombobox.h>
 #include <qstatusbar.h>
+#include <qeventloop.h>
 
 
 #include "DialogFactory.h"
 #include "FWBTree.h"
 #include "FWWindow.h"
+#include "ConfirmDeleteObjectDialog.h"
 
 #include "fwbuilder/Library.h"
 #include "fwbuilder/Firewall.h"
@@ -339,6 +341,9 @@ void ObjectManipulator::showDeletedObjects(bool f)
 
 void ObjectManipulator::removeObjectFromTreeView(FWObject *obj )
 {
+    QListView *objTreeView = idxToTrees[ getIdxForLib(getCurrentLib()) ];
+    dynamic_cast<ObjectTreeView*>(objTreeView)->clearLastSelected();
+
     ObjectTreeViewItem *itm = allItems[obj];
     allItems[obj]=NULL;
 //    allItems.erase(obj);
@@ -462,7 +467,7 @@ void ObjectManipulator::updateObjName(FWObject *obj,
          (QString::fromUtf8(obj->getName().c_str())!=oldName))
     {
         //mw->reopenFirewall();
-        QTimer::singleShot( 0, mw, SLOT(reopenFirewall()) );
+        mw->scheduleRuleSetRedraw();
     }
 
     info();  // need to update info in case user edited comments and other attributes.
@@ -958,6 +963,7 @@ void ObjectManipulator::contextMenu(QListViewItem *item,
 
         popup->insertSeparator();
         popup->insertItem( tr("Find") , this , SLOT( findObject()));
+        popup->insertItem( tr("Where used") , this , SLOT( findWhereUsedSlot()));
 /*
         if (Firewall::cast(currentObj)!=NULL)
         {
@@ -992,11 +998,12 @@ void ObjectManipulator::contextMenu(QListViewItem *item,
     popup->setItemEnabled(lcID,  getCurrentObjectTree()->isLockable());
     popup->setItemEnabled(unlcID,  getCurrentObjectTree()->isUnlockable());
     
-#if 0
+    if (fwbdebug)
+    {
 /* keep this for debugging  */
-    popup->insertSeparator();
-    popup->insertItem( tr("dump") , this , SLOT( dumpObj()));
-#endif
+        popup->insertSeparator();
+        popup->insertItem( tr("dump") , this , SLOT( dumpObj()));
+    }
 
     if (getCurrentObjectTree()->getNumSelected()==1)
     {
@@ -1072,11 +1079,11 @@ void ObjectManipulator::getMenuState(bool haveMoveTargets,
             /*
              *  enable Paste menu item only if object can be pasted
              */
-            vector<FWObject*>::iterator i;
+            vector<string>::iterator i;
             for (i= FWObjectClipboard::obj_clipboard->begin();
                  i!=FWObjectClipboard::obj_clipboard->end(); ++i)
             {
-                FWObject *co= *i;
+                FWObject *co= mw->db()->findInIndex(*i);
                 FWObject *nobj=pasteTo( obj , co , false, true);
                 pasteMenuItem = pasteMenuItem && (nobj!=NULL);
             }
@@ -1155,7 +1162,7 @@ void ObjectManipulator::dumpObj()
 
     FWObject *obj=getCurrentObjectTree()->getSelectedObjects().front();
     if (obj==NULL) return;
-    obj->dump(true,true);
+    obj->dump(false,false);
 }
 
 void ObjectManipulator::compile()
@@ -1482,11 +1489,11 @@ void ObjectManipulator::pasteObj()
     FWObject *obj=getCurrentObjectTree()->getSelectedObjects().front();
     if (obj==NULL) return;
 
-    vector<FWObject*>::iterator i;
+    vector<string>::iterator i;
     for (i= FWObjectClipboard::obj_clipboard->begin();
          i!=FWObjectClipboard::obj_clipboard->end(); ++i)
     {
-        FWObject *co= *i;
+        FWObject *co= mw->db()->findInIndex(*i);
         FWObject *nobj=pasteTo( obj , co );
         if (nobj!=NULL)
         {
@@ -1538,7 +1545,7 @@ FWObject*  ObjectManipulator::pasteTo(FWObject *target,FWObject *obj,
 /* add a copy of the object to system group */
 
             FWObject *nobj=
-                mw->db()->create(obj->getTypeName(),true);
+                mw->db()->create(obj->getTypeName());
             assert (nobj!=NULL);
             nobj->ref();
             nobj->duplicate(obj,true);   // creates new object ID
@@ -1689,6 +1696,7 @@ void ObjectManipulator::deleteObj()
     {
         if (!emptyingTrash && (so.size()>1 || !Library::isA(so.front())))
         {
+            /*
             msg = tr(
                 "When you delete an object, it is removed from the tree and\n"
                 "all groups and firewall policy rules that reference it.\n"
@@ -1698,6 +1706,14 @@ void ObjectManipulator::deleteObj()
                     this,"Firewall Builder", msg,
                     tr("&Yes"), tr("&No"), QString::null,
                     0, 1 )!=0) return;
+                    */
+            QApplication::setOverrideCursor( QCursor( Qt::WaitCursor) );
+            ConfirmDeleteObjectDialog * dlg= new ConfirmDeleteObjectDialog(this);
+            
+            dlg->load(so);
+
+            QApplication::restoreOverrideCursor();
+            if(dlg->exec()==QDialog::Rejected ) return;
         }
     }
 
@@ -1814,39 +1830,6 @@ void ObjectManipulator::delObj(FWObject *obj,bool openobj)
 
 //        removeObjectFromTreeView(obj);
 
-#if 0
-        QApplication::setOverrideCursor( QCursor( Qt::WaitCursor) );
-        if (islib && !isDelObj)
-        {
-            if (obj->getId()==TEMPLATE_LIB) // special case
-            {
-                if (fwbdebug) qDebug("ObjectManipulator::delObj:   special case: deleting template library");
-                if (obj->isReadOnly()) obj->setReadOnly(false);
-                mw->db()->removeAllInstances(obj);
-            } else
-            {
-                if (fwbdebug) qDebug("ObjectManipulator::delObj:   recursively deleting library and all its objects");
-                if (obj->isReadOnly()) obj->setReadOnly(false);
-                mw->db()->recursivelyRemoveObjFromTree(obj, false);
-
-                parent=mw->db()->getFirstByType(Library::TYPENAME);
-            }
-        } else
-        {
-            if (obj->getId()==TEMPLATE_LIB) // special case
-            {
-                if (fwbdebug) qDebug("ObjectManipulator::delObj:   special case: deleting template library");
-                mw->db()->removeAllInstances(obj);
-            } else
-            {
-                if (fwbdebug) qDebug("ObjectManipulator::delObj:   recursively deleting object");
-                mw->db()->recursivelyRemoveObjFromTree(obj, false);
-            }
-        }
-        QApplication::restoreOverrideCursor();
-        if (fwbdebug) qDebug("ObjectManipulator::delObj:   done");
-#endif
-
         QApplication::setOverrideCursor( QCursor( Qt::WaitCursor) );
 
         if (islib && obj->isReadOnly()) obj->setReadOnly(false);
@@ -1859,7 +1842,6 @@ void ObjectManipulator::delObj(FWObject *obj,bool openobj)
         {
             if (fwbdebug) qDebug("ObjectManipulator::delObj:   recursively deleting library and all its objects");
             mw->db()->recursivelyRemoveObjFromTree(obj, false);
-
             if (islib) parent=mw->db()->getFirstByType(Library::TYPENAME);
         }
 
@@ -1867,18 +1849,32 @@ void ObjectManipulator::delObj(FWObject *obj,bool openobj)
         if (fwbdebug) qDebug("ObjectManipulator::delObj:   done");
 
         removeObjectFromTreeView(obj);
+        mw->scheduleRuleSetRedraw();
 
         if (!isDelObj)
         {
             if (allItems[delObjLib]!=NULL)
                 insertSubtree( allItems[delObjLib], obj );
+        } else
+            FWObjectClipboard::obj_clipboard->clear();
+
+        if (openobj)
+        {
+            if (isfw)
+            {
+                std::list<Firewall*> fwlist;
+                findAllFirewalls(fwlist);
+                FWObject *first_fw = fwlist.front();
+                if (first_fw!=NULL)
+                {
+                    mw->showFirewall( first_fw );
+                    openObject( first_fw );
+                }
+                //QTimer::singleShot( 0, mw, SLOT(reopenFirewall()) );
+            } else {
+                openObject(parent);
+            }
         }
-
-        //mw->reopenFirewall();
-        QTimer::singleShot( 0, mw, SLOT(reopenFirewall()) );
-
-        if (openobj) openObject(parent);
-
     }
     catch(FWException &ex)
     {
@@ -2325,7 +2321,7 @@ FWObject* ObjectManipulator::actuallyCreateObject(FWObject *parent,
                                                   FWObject *copyFrom)
 {
     if (!isTreeReadWrite(this, parent)) return NULL;
-    FWObject *nobj = mw->db()->create(objType.latin1(),true);
+    FWObject *nobj = mw->db()->create(objType.latin1());
     assert(nobj!=NULL);
 
     if (copyFrom!=NULL) nobj->duplicate(copyFrom,true);
@@ -2815,6 +2811,7 @@ void ObjectManipulator::updateLastModifiedTimestampForAllFirewalls(FWObject *o)
 
     QStatusBar *sb = mw->statusBar();
     sb->message( tr("Searching for firewalls affected by the change...") );
+    QApplication::eventLoop()->processEvents(QEventLoop::ExcludeUserInput,100);
 
     if (fwbdebug) qDebug("ObjectManipulator::updateLastModifiedTimestampForAllFirewalls:   setOverrideCursor");
     QApplication::setOverrideCursor(QCursor( Qt::WaitCursor));
@@ -2837,6 +2834,7 @@ void ObjectManipulator::updateLastModifiedTimestampForAllFirewalls(FWObject *o)
     if (fwbdebug) qDebug("ObjectManipulator::updateLastModifiedTimestampForAllFirewalls:   restoreOverrideCursor");
     QApplication::restoreOverrideCursor();
     sb->clear();
+    QApplication::eventLoop()->processEvents(QEventLoop::ExcludeUserInput,100);
 }
 
 void ObjectManipulator::updateLastInstalledTimestamp(FWObject *o)
@@ -2867,6 +2865,7 @@ void ObjectManipulator::updateLastCompiledTimestamp(FWObject *o)
         info();
     }
 }
+
 void ObjectManipulator::simulateInstall()
 {
     if (fwbdebug) qDebug("ObjectManipulator::simulateInstall");
@@ -2888,23 +2887,17 @@ void ObjectManipulator::simulateInstall()
     getCurrentObjectTree()->updateTreeItems (); 
     
 }
-void ObjectManipulator::findFirewallsInCurrentLib (list<Firewall *> &fws)
+
+void ObjectManipulator::findAllFirewalls (list<Firewall *> &fws)
 {
-    Library *lib= Library::cast(getCurrentLib());
-    // fg: Firewalls (Group)
-    FWObject *fg=lib->findObjectByName(ObjectGroup::TYPENAME,"Firewalls");
-    
-    FWObject::iterator i=fg->begin();
-    for ( ;i!=fg->end();++i)
-    {
-        Firewall *f=Firewall::cast(*i);
-        if (f!=NULL)
-        {
-            fws.push_back(f);
-        }
-    }
-    
+    if (fwbdebug) qDebug("ObjectManipulator::findAllFirewalls");
+
+    std::list<FWObject*> fwlist;
+    findByObjectType(mw->db(),Firewall::TYPENAME,fwlist);
+    for (list<FWObject*>::iterator m=fwlist.begin(); m!=fwlist.end(); m++)
+        fws.push_back( Firewall::cast(*m) );
 }
+
 void ObjectManipulator::findObjectsInGroup(Group *g,set<FWObject *> &res)
 {
     if (fwbdebug) qDebug("ObjectManipulator::findObjectsInGroup");
@@ -2949,3 +2942,13 @@ FWObject* ObjectManipulator::getSelectedObject()
 {
     return currentObj;
 }
+void ObjectManipulator::findWhereUsedSlot()
+{
+    if (getCurrentObjectTree()->getNumSelected()==0) return;
+
+    FWObject *obj=getCurrentObjectTree()->getSelectedObjects().front();
+    if (obj==NULL) return;
+    mw->findWhereUsed(obj);
+    
+}
+
