@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: ObjectManipulator.h,v 1.53 2006/06/12 06:55:05 vkurland Exp $
+  $Id: ObjectManipulator.h,v 1.56 2006/08/18 04:10:33 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -47,8 +47,11 @@ class ObjectTreeView;
 class ObjectTreeViewItem;
 class QComboBox;
 class QPopupMenu;
-class libfwbuilder::Firewall;
-
+namespace libfwbuilder 
+{
+    class Firewall;
+    class Library;
+}
 class HistoryItem {
     ObjectTreeViewItem *itm;
     QString         objId;
@@ -65,7 +68,7 @@ class ObjToolTip : public QToolTip {
     ObjectTreeView *otv;
  public:
     ObjToolTip(ObjectTreeView *widget);
-
+    virtual ~ObjToolTip() {}
     virtual void maybeTip(const QPoint &p);
 };
 
@@ -207,6 +210,7 @@ public slots:
      virtual void lockObject();
      virtual void unlockObject();
      virtual void simulateInstall();
+     virtual void findWhereUsedSlot();
 
      
  public:
@@ -328,8 +332,9 @@ public slots:
             std::set<libfwbuilder::FWObject *> &resset);
     
     std::list<libfwbuilder::Firewall * > findFirewallsForObject(libfwbuilder::FWObject *o);
-    void findFirewallsInCurrentLib (std::list<libfwbuilder::Firewall *> &fws);
+    void findAllFirewalls (std::list<libfwbuilder::Firewall *> &fws);
 
+    
  signals:
 /**
  * the dialog class should have a slot that can load object's data
