@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: ObjectTreeView.cpp,v 1.34 2006/07/10 00:26:08 vkurland Exp $
+  $Id: ObjectTreeView.cpp,v 1.35 2006/09/15 05:48:21 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -487,10 +487,14 @@ void ObjectTreeView::contentsMousePressEvent( QMouseEvent *e )
     second_click = false;
 
     if (fwbdebug)
-        qDebug(QString("ObjectTreeView::contentsMousePressEvent :: currentItem=%1 lastSelected=%2")
+    {
+        qDebug(QString("ObjectTreeView::contentsMousePressEvent :: currentItem=%1")
                .arg((currentItem())?currentItem()->text(0):"nil")
+        );
+        qDebug(QString("ObjectTreeView::contentsMousePressEvent :: lastSelected=%2")
                .arg((lastSelected)?lastSelected->text(0):"nil")
         );
+    }
 
     lastSelected = currentItem();
 
@@ -625,6 +629,12 @@ void ObjectTreeView::returnPressed(QListViewItem *itm)
     QListView::returnPressed(itm);
 }
 
+void ObjectTreeView::clearLastSelected()
+{
+    lastSelected = NULL;
+}
+
+
 void ObjectTreeView::resetSelection()
 {
     if (fwbdebug)
