@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: PrefsDialog.cpp,v 1.25 2004/12/17 08:04:14 vkurland Exp $
+  $Id: PrefsDialog.cpp,v 1.27 2006/06/26 05:35:47 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -96,9 +96,8 @@ PrefsDialog::PrefsDialog(QWidget *parent) : prefsDialog_q(parent)
     autosaveFile->setChecked( st->getBool("Environment/autoSaveFile") );
     autosaveInterval->setValue( st->getInt("Environment/autoSaveFilePeriod"));
 
-    dontSaveStdLib->setChecked( st->getDontSaveStdLib() );
+//    dontSaveStdLib->setChecked( st->getDontSaveStdLib() );
 
-    scpPath->setText( st->getSCPPath() );
     sshPath->setText( st->getSSHPath() );
 
     for (list<libData>::iterator i=addOnLibs->begin(); i!=addOnLibs->end(); ++i)
@@ -215,15 +214,6 @@ void PrefsDialog::findWDir()
     if (!dir.isEmpty()) wDir->setText(dir);
 }
 
-void PrefsDialog::findSCP()
-{
-    QString fp = QFileDialog::getOpenFileName(
-        QString::null, QString::null, this,
-        tr("Find Secure File Transfer utility") );
-
-    if (!fp.isEmpty()) scpPath->setText(fp);
-}
-
 void PrefsDialog::findSSH()
 {
     QString fp = QFileDialog::getOpenFileName(
@@ -331,7 +321,7 @@ void PrefsDialog::accept()
     st->setBool("Environment/autoSaveFile", autosaveFile->isChecked() );
     st->setInt("Environment/autoSaveFilePeriod", autosaveInterval->value() );
 
-    st->setDontSaveStdLib( dontSaveStdLib->isChecked() );
+//    st->setDontSaveStdLib( dontSaveStdLib->isChecked() );
 
     st->setLabelColor(FWBSettings::RED,    colors[FWBSettings::RED]);
     st->setLabelColor(FWBSettings::ORANGE, colors[FWBSettings::ORANGE]);
@@ -349,7 +339,6 @@ void PrefsDialog::accept()
     st->setLabelText (FWBSettings::PURPLE, purpleText->text() );
     st->setLabelText (FWBSettings::GRAY,   grayText->text() );
 
-    st->setSCPPath( scpPath->text() );
     st->setSSHPath( sshPath->text() );
 
     QDir d;
