@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: RuleSetView.cpp,v 1.144 2006/07/23 15:48:02 vkurland Exp $
+  $Id: RuleSetView.cpp,v 1.150 2006/09/15 05:48:21 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -304,6 +304,29 @@ void RuleSetView::setRuleNumber(int row, libfwbuilder::Rule *rule)
     verticalHeader()->resizeSection( row , s );
 }
 
+void RuleSetView::fixRulePosition(Rule *rule, FWObject *parent, int pos)
+{
+    if ( rule->isReadOnly())
+    {
+        // need to temporary break the lock
+        // since several parents could be read-only, do it recursively
+        FWObject *o = (parent!=NULL) ? parent : rule;
+        while ( o!=NULL && !o->getBool("ro") ) o = o->getParent();
+        if (o)
+        {
+            o->setReadOnly(false);
+            fixRulePosition(rule, o, pos);
+            o->setReadOnly(true);
+        } else
+        {
+            rule->checkReadOnly();   // should be read-write by now
+            rule->setPosition(pos);
+        }
+    } else
+        rule->setPosition(pos);
+}
+
+
 void RuleSetView::init()
 {
     QApplication::setOverrideCursor( QCursor( Qt::WaitCursor) );
@@ -342,8 +365,9 @@ void RuleSetView::init()
 
         setRuleNumber(row, Rule::cast( *i ));
         if (Rule::cast( *i )->getPosition()!=row)
-            Rule::cast( *i )->setPosition(row);
-
+        {
+            fixRulePosition(Rule::cast( *i ), NULL, row);
+        }
 //        adjustRow(row);
 
         int h=20;
@@ -495,7 +519,12 @@ void RuleSetView::adjustRow( int row )
 
 QRect RuleSetView::calculateCellSize( int row, int col )
 {
-    int   h = 20;
+    int h = 20;
+    int re_size;
+
+//    if (fwbdebug)
+//        qDebug("RuleSetView::calculateCellSize: row=%d col=%d",
+//               row,col);
 
     QPainter p(this);
 
@@ -511,7 +540,7 @@ QRect RuleSetView::calculateCellSize( int row, int col )
         if (re==NULL)
         {
 /* broken rule element, fix it */
-            FWObject *nre=mw->db()->create("When",true);
+            FWObject *nre=mw->db()->create("When");
             assert(nre!=NULL);
             rule->add(nre);
         }
@@ -522,6 +551,7 @@ QRect RuleSetView::calculateCellSize( int row, int col )
     {
         RuleElement *re = getRE(rule,col);
         if (re==NULL) return QRect(0,0,0,0);
+        re_size = re->size();
         for (FWObject::iterator j=re->begin(); j!=re->end(); j++)
         {
             FWObject *o1= *j;
@@ -664,6 +694,8 @@ void RuleSetView::paintCell(QPainter *pntr,
                             bool selected,
                             const QColorGroup &cg)
 {
+    int re_size;
+
 /* row may point at an empty row where there is no rule yet. This
  * happens if this method is called to redraw the table when we call
  * setNumRows
@@ -727,7 +759,7 @@ void RuleSetView::paintCell(QPainter *pntr,
     {
         RuleElement *re = getRE(row,col);
         if (re==NULL) return;
-
+        re_size = re->size();
         for (FWObject::iterator i=re->begin(); i!=re->end(); i++)
         {
             FWObject *o1= *i;
@@ -763,7 +795,7 @@ void RuleSetView::paintCell(QPainter *pntr,
             
             FWObject *mwSelObj=selectedObject;
             if (mwSelObj==NULL) mwSelObj=om->getCurrentObjectTree()->getSelectedObjects().front();
-            if (    !sel &&
+            if (    (!sel || numSelections()==0) &&
                     mwSelObj!= NULL &&
                     mwSelObj->getId() != "sysid0" &&
                     mwSelObj->getId() != "sysid1" &&
@@ -979,7 +1011,8 @@ void RuleSetView::selectRE(libfwbuilder::FWReference *ref)
 void RuleSetView::unselect()
 {
 //    selectedObject = NULL;
-    setCurrentCell(-1,-1);
+//    setCurrentCell(-1,-1);
+    clearSelection();
     selectedObject=NULL;
     updateCell(currentRow(),currentColumn());
     updateContents();
@@ -1699,6 +1732,7 @@ void RuleSetView::setRuleColor(const QString &c)
         }
     }
 }
+
 void RuleSetView::changeAction(PolicyRule::Action act)
 {
     if (!isTreeReadWrite(this,ruleset)) return;
@@ -1719,6 +1753,7 @@ void RuleSetView::changeAction(PolicyRule::Action act)
         oe->actionChanged(rule);    
     }
 }
+
 void RuleSetView::changeActionToAccept()
 {
     if (!isTreeReadWrite(this,ruleset)) return;
@@ -1934,8 +1969,17 @@ void RuleSetView::deleteObject(int row, int col, FWObject *obj)
 {
     RuleElement *re = getRE(row,col);
     if (re==NULL || re->isAny()) return;
-    
-    if (fwbdebug) qDebug("Firewall changed: deleteObject"); 
+    string id = obj->getId();
+
+    if (fwbdebug)
+    {
+        qDebug("RuleSetView::deleteObject row=%d col=%d id=%s",
+               row,col,id.c_str()); 
+        qDebug("obj = %p",re->getRoot()->findInIndex(id));
+        int rc = obj->ref()-1;  obj->unref();
+        qDebug("obj->ref_counter=%d",rc);
+    }
+
     if (!changingRules)
         om->updateLastModifiedTimestampForOneFirewall(getFirewall());
     
@@ -1943,6 +1987,14 @@ void RuleSetView::deleteObject(int row, int col, FWObject *obj)
 
     if (re->isAny()) re->setNeg(false);
 
+    if (fwbdebug)
+    {
+        qDebug("RuleSetView::deleteObject re->size()=%d",re->size()); 
+        qDebug("obj = %p",re->getRoot()->findInIndex(id));
+        int rc = obj->ref()-1;  obj->unref();
+        qDebug("obj->ref_counter=%d",rc);
+    }
+
     dirtyRows[row]=1;
     adjustColumn(col);
     adjustRow(row);
@@ -2030,11 +2082,11 @@ void RuleSetView::pasteObject()
 {
     if (!isTreeReadWrite(this,ruleset)) return;
 
-    vector<FWObject*>::iterator i;
+    vector<string>::iterator i;
     for (i= FWObjectClipboard::obj_clipboard->begin();
          i!=FWObjectClipboard::obj_clipboard->end(); ++i)
     {
-        FWObject *co= *i;
+        FWObject *co= mw->db()->findInIndex(*i);
         if (Rule::cast(co)!=NULL)  pasteRuleAbove();
         else
         {
@@ -2933,11 +2985,11 @@ void RuleSetView::pasteRuleAbove()
     if (fwbdebug) qDebug("Firewall: pasteRuleAbove"); 
     
 /* pick rules in reverse order */
-    vector<FWObject*>::reverse_iterator i;
+    vector<string>::reverse_iterator i;
     for (i= FWObjectClipboard::obj_clipboard->rbegin();
          i!=FWObjectClipboard::obj_clipboard->rend(); ++i)
     {
-        FWObject *co= *i;
+        FWObject *co= mw->db()->findInIndex(*i);
         if (!Rule::cast(co)) continue;
         insertRule( firstSelectedRule, co);
     }
@@ -2970,11 +3022,11 @@ void RuleSetView::pasteRuleBelow()
     }
 
     int n=0;
-    vector<FWObject*>::iterator i;
+    vector<string>::iterator i;
     for (i= FWObjectClipboard::obj_clipboard->begin();
          i!=FWObjectClipboard::obj_clipboard->end(); ++i,++n)
     {
-        FWObject *co= *i;
+        FWObject *co= mw->db()->findInIndex(*i);
         if (!Rule::cast(co)) continue;
         insertRule( lastSelectedRule+1+n, co);
     }
