@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: instDialog.cpp,v 1.85 2006/07/23 22:40:06 vkurland Exp $
+  $Id: instDialog.cpp,v 1.94 2006/09/17 05:15:50 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -39,7 +39,6 @@
 
 #include <qcheckbox.h>
 #include <qlineedit.h>
-#include <qtextbrowser.h>
 #include <qtextedit.h>
 #include <qtimer.h>
 #include <qfiledialog.h>
@@ -56,6 +55,7 @@
 #include <qcolor.h>
 #include <qtable.h>
 #include <qtextcodec.h>
+#include <qfileinfo.h>
 
 #include "fwbuilder/Resources.h"
 #include "fwbuilder/FWObjectDatabase.h"
@@ -77,9 +77,11 @@ using namespace libfwbuilder;
 
 
 
-instDialog::instDialog(QWidget* p, BatchOperation op):instDialog_q(p)
+instDialog::instDialog(QWidget* p, BatchOperation op) : instDialog_q(p)
 {
-    if (fwbdebug) qDebug("instDialog::instDialog");
+    if (fwbdebug) qDebug("instDialog::instDialog   WFlagse=%0x",getWFlags());
+
+    clearWFlags( Qt::WType_Dialog | Qt::WStyle_SysMenu  );
     
     session=NULL;
     dlg=NULL;
@@ -89,7 +91,8 @@ instDialog::instDialog(QWidget* p, BatchOperation op):instDialog_q(p)
     currentSaveButton = NULL;    
     currentStopButton = NULL;
     showSelectedFlag=false;
-    
+    pendingLogLine = "";
+
     proc.setCommunication( QProcess::Stdout|QProcess::Stderr|QProcess::DupStderr);
 
     connect(&proc, SIGNAL(readyReadStdout()), this, SLOT(readFromStdout()) );
@@ -150,6 +153,8 @@ instDialog::instDialog(QWidget* p, BatchOperation op):instDialog_q(p)
         detailMCframe->show();
     else
         detailMCframe->hide();
+
+    togleDetailMC();
 }
 
 instDialog::~instDialog()
@@ -195,10 +200,14 @@ void instDialog::prepareInstallerOptions(bool fPix)
 
     dlg->pwd->setEchoMode( QLineEdit::Password );
     dlg->epwd->setEchoMode( QLineEdit::Password );
-    
+
+    QString fwname = QString::fromUtf8((*opListIterator)->getName().c_str());
     if (!batchInstall->isChecked())
-        dlg->textLabel1_4->setText(dlg->textLabel1_4->text().arg(QString::fromUtf8(
-                    (*opListIterator)->getName().c_str())));
+        dlg->dialogTitleLine->setText(
+            QString("<p align=\"center\"><b><font size=\"+2\">")+
+            tr("Install options for firewall '%1'").arg(fwname)+
+            QString("</font></b></p>")
+        );
     cnf.incremental=st->readBoolEntry("/FirewallBuilder2/Installer/incr"    );
     cnf.save_diff  =st->readBoolEntry("/FirewallBuilder2/Installer/savediff");
     cnf.saveStandby=st->readBoolEntry("/FirewallBuilder2/Installer/saveStandby");
@@ -207,6 +216,7 @@ void instDialog::prepareInstallerOptions(bool fPix)
     cnf.verbose    =st->readBoolEntry("/FirewallBuilder2/Installer/verbose" );
     cnf.stripComments=st->readBoolEntry("/FirewallBuilder2/Installer/stripComments" );
     cnf.compressScript=st->readBoolEntry("/FirewallBuilder2/Installer/compressScript" );
+    cnf.copyFWB     =st->readBoolEntry("/FirewallBuilder2/Installer/copyFWB" );
     cnf.testRun     =st->readBoolEntry("/FirewallBuilder2/Installer/testRun" );
     cnf.rollback    =st->readBoolEntry("/FirewallBuilder2/Installer/rollback" );
     cnf.rollbackTime=st->readNumEntry("/FirewallBuilder2/Installer/rollbackTime");
@@ -218,30 +228,33 @@ void instDialog::prepareInstallerOptions(bool fPix)
         Resources::getTargetOptionStr(cnf.fwobj->getStr("host_OS"),
                                       "activation/timeout_units").c_str();
     if ((platform!="pix" && platform!="fwsm" && !batchInstall->isChecked()) ||
-           (!fPix && batchInstall->isChecked()) )
+        (!fPix && batchInstall->isChecked()) )
     {
-       dlg->incr->hide();
-       dlg->test->hide();
-       dlg->saveDiff->hide();
-       dlg->saveStandby->hide();
-       dlg->backupConfigFile->hide();
-       dlg->backupConfigFileLbl->hide();
-       dlg->epwd->hide();
-       dlg->epwdLbl->hide();
+        dlg->incr->hide();
+        dlg->test->hide();
+        dlg->saveDiff->hide();
+        dlg->saveStandby->hide();
+        dlg->backupConfigFile->hide();
+        dlg->backupConfigFileLbl->hide();
+        dlg->epwd->hide();
+        dlg->epwdLbl->hide();
 
-       dlg->PIXgroupBox->hide();
+        dlg->PIXgroupBox->hide();
     } else
     {
-       dlg->incr->show();
-       dlg->test->show();
-       dlg->saveDiff->show();
-       dlg->saveStandby->show();
-       dlg->backupConfigFile->show();
-       dlg->backupConfigFileLbl->show();
-       dlg->epwd->show();
-       dlg->epwdLbl->show();
+// if PIX or FWSM:
+        dlg->copyFWB->hide();
 
-       dlg->PIXgroupBox->show();
+        dlg->incr->show();
+        dlg->test->show();
+        dlg->saveDiff->show();
+        dlg->saveStandby->show();
+        dlg->backupConfigFile->show();
+        dlg->backupConfigFileLbl->show();
+        dlg->epwd->show();
+        dlg->epwdLbl->show();
+
+        dlg->PIXgroupBox->show();
     }
 
 /* hide anyway, diff does not work for pix 6.3(3) */
@@ -275,6 +288,7 @@ void instDialog::prepareInstallerOptions(bool fPix)
    dlg->verbose->setChecked( cnf.verbose );
    dlg->stripComments->setChecked( cnf.stripComments );
    dlg->compressScript->setChecked( cnf.compressScript );
+   dlg->copyFWB->setChecked( cnf.copyFWB );
    dlg->testRun->setChecked( cnf.testRun );
    dlg->rollback->setChecked( cnf.rollback );
    dlg->rollbackTime->setValue( cnf.rollbackTime );
@@ -346,6 +360,7 @@ void instDialog::prepareInstallerOptions(bool fPix)
     }
     
 }
+
 void instDialog::storeInstallerOptions()
 {
     st->writeEntry("/FirewallBuilder2/Installer/incr",    cnf.incremental);
@@ -356,21 +371,23 @@ void instDialog::storeInstallerOptions()
     st->writeEntry("/FirewallBuilder2/Installer/verbose", cnf.verbose    );
     st->writeEntry("/FirewallBuilder2/Installer/stripComments", cnf.stripComments);
     st->writeEntry("/FirewallBuilder2/Installer/compressScript", cnf.compressScript);
+    st->writeEntry("/FirewallBuilder2/Installer/copyFWB",  cnf.copyFWB);
     st->writeEntry("/FirewallBuilder2/Installer/testRun",  cnf.testRun);
     st->writeEntry("/FirewallBuilder2/Installer/rollback", cnf.rollback);
     st->writeEntry("/FirewallBuilder2/Installer/rollbackTime", cnf.rollbackTime);
 }
+
 void instDialog::append(const QString &s)
 {
-    currentLog->moveCursor( QTextEdit::MoveEnd , false );
-    currentLog->insert( s );
-//    progressDisplay->append(s);
+//    currentLog->moveCursor( QTextEdit::MoveEnd , false );
+//    currentLog->insert( s );
+
+    currentLog->append( s );
 }
 
 void instDialog::appendRich(const QString &s)
 {
     if (currentLog) currentLog->append(s);
-//    progressDisplay->insertParagraph(s,-1);
 }
 
 
@@ -431,7 +448,9 @@ void instDialog::fillCompileSelectList()
     Firewall* f;
     QDateTime dt;
     int row=0;
-
+    
+    bool show_library=false;
+    string tmp_libname="";
     
     for (std::list<libfwbuilder::Firewall *>::iterator i=firewalls.begin();
             i!=firewalls.end(); ++i)
@@ -444,6 +463,16 @@ void instDialog::fillCompileSelectList()
         
         selectTable->setText(row,2,QString::fromUtf8(f->getName().c_str()));
         selectTable->setColumnReadOnly(2,true);
+
+        // in fact, if someone use same names for several libraries,
+        // additional collumn with library names doesn't help to
+        // identify a firewall
+        if (!show_library && tmp_libname != "" && tmp_libname != f->getLibraryName())
+            show_library = true;
+        tmp_libname = f->getLibraryName();
+        
+        selectTable->setText(row,3,QString::fromUtf8(tmp_libname.c_str()));
+        selectTable->setColumnReadOnly(3,true);
         
         citem=new QCheckTableItem(selectTable,"");
         citem->setChecked(
@@ -470,28 +499,32 @@ void instDialog::fillCompileSelectList()
         l->setBackgroundColor(Qt::white);
         if (lm>lc && lm>li) {font=l->font();font.setBold(true);l->setFont(font);}
         l->setText((lm)?dt.toString():QString("---"));
-        selectTable->setCellWidget(row,3,l);
+        selectTable->setCellWidget(row,4,l);
         
         dt.setTime_t(lc);
         l=new QLabel(selectTable);
         l->setBackgroundColor(Qt::white);
         if (lc>lm && lc>li) {font=l->font();font.setBold(true);l->setFont(font);}
         l->setText((lm)?dt.toString():QString("---"));
-        selectTable->setCellWidget(row,4,l);
+        selectTable->setCellWidget(row,5,l);
         
         dt.setTime_t(li);
         l=new QLabel(selectTable);
         l->setBackgroundColor(Qt::white);
         if (li>lc && li>lm) {font=l->font();font.setBold(true);l->setFont(font);}
         l->setText((lm)?dt.toString():QString("---"));
-        selectTable->setCellWidget(row,5,l);
+        selectTable->setCellWidget(row,6,l);
         
         row++;        
     }
-    
+    if (show_library)
+        selectTable->showColumn(3);
+    else
+        selectTable->hideColumn(3);
+       
     for (int i=0;i<selectTable->numCols();i++)
     {
-        if (i<3)            
+        if (i<4)            
             selectTable->adjustColumn(i);
         else
             selectTable->setColumnWidth(i,200);
@@ -639,6 +672,7 @@ bool instDialog::doInstallPage()
         cnf.verbose       = dlg->verbose->isChecked();
         cnf.stripComments = dlg->stripComments->isChecked();
         cnf.compressScript= dlg->compressScript->isChecked();
+        cnf.copyFWB       = dlg->copyFWB->isChecked();
         cnf.testRun       = dlg->testRun->isChecked();
         cnf.rollback      = dlg->rollback->isChecked();
         cnf.rollbackTime  = dlg->rollbackTime->value();
@@ -656,14 +690,14 @@ bool instDialog::doInstallPage()
         }
         if (nmi>1)
         {
-	   appendRich(
+	   addToLog(
 		QObject::tr("Only one interface of the firewall '%1' must be marked as management interface.")
 		.arg(cnf.fwobj->getName().c_str()).latin1() );
 	    return false;
         }
         if (nmi==0)
         {
-	   appendRich(
+	   addToLog(
 		QObject::tr("One of the interfaces of the firewall '%1' must be marked as management interface.")
 		.arg(cnf.fwobj->getName().c_str()).latin1() );
 	    return false;
@@ -671,16 +705,24 @@ bool instDialog::doInstallPage()
         if ((cnf.maddr == "" || cnf.maddr == "0.0.0.0") &&
             dlg->altAddress->text().isEmpty())
 	{
-	   appendRich(
+	   addToLog(
                QObject::tr("Management interface does not have IP address, can not communicate with the firewall.") );
 	    return false;
 	}
 
         confScript="";
 
-/* read manifest from the conf file */
-
         confFiles.clear();
+        if (cnf.copyFWB)
+        {
+            QFileInfo fwbfile_base(cnf.fwbfile);
+            
+            if (fwbdebug)
+                qDebug( QString("Will copy data file: %1").arg(fwbfile_base.fileName()));
+
+            confFiles.push_back( fwbfile_base.fileName() );
+        }
+/* read manifest from the conf file */
 
         QFile cf(  cnf.wdir+"/"+cnf.conffile );
         if (cf.open( IO_ReadOnly ) )
@@ -767,7 +809,8 @@ bool instDialog::doInstallPage()
 
             phase=1;
 
-            runSSH( new SSHPIX(cnf.fwobj->getName().c_str(),
+            runSSH( new SSHPIX(this,
+                               cnf.fwobj->getName().c_str(),
                                args,
                                cnf.pwd,
                                cnf.epwd,
@@ -797,15 +840,21 @@ bool instDialog::doInstallPage()
         }
         return true;
 }
-void instDialog::doOptionsPage()
+
+void instDialog::resetInstallSSHSession()
 {
+    if (fwbdebug) qDebug("instDialog::resetInstallSSHSession");
+
     if (session!=NULL)
     {
         disconnect(session,SIGNAL(printStdout_sign(const QString&)),
                    this,SLOT(append(const QString&)));
 
         disconnect(session,SIGNAL(sessionFinished_sign()),
-                   this,SLOT(processExited()));
+                   this,SLOT(installerFinished()));
+
+        disconnect(session,SIGNAL(sessionFatalError_sign()),
+                   this,SLOT(installerError()));
 
         disconnect(session,SIGNAL(updateProgressBar_sign(int,bool)),
                    this,SLOT(updateProgressBar(int,bool)));
@@ -817,7 +866,6 @@ void instDialog::doOptionsPage()
     }
 
     activationCommandDone=false;
-
 }
 
 /*
@@ -846,7 +894,7 @@ QString instDialog::getActivationCmd()
     } else
     {
         optpath += "run/";
-        if (cnf.compressScript) optpath += "compression/";
+        if (cnf.compressScript)  optpath += "compression/";
         else                     optpath += "no_compression/";
     }
 
@@ -882,17 +930,23 @@ void instDialog::initiateCopy(const QString &file)
     QString platform=cnf.fwobj->getStr("platform").c_str();
     //if (platform!="pix" && platform!="fwsm") progressBar->show();
 
-    QString ff = cnf.wdir+"/"+file;
-
     QTextCodec::setCodecForCStrings(QTextCodec::codecForName("latin1"));
     
-    std::ifstream   *config_file;
+    std::ifstream   *wfile;
 
-    config_file = new ifstream(ff.latin1());
-    if ( ! *config_file)
+    QString ff = cnf.wdir+"/"+file;
+
+    wfile = new ifstream(ff.latin1());
+    if ( ! *wfile)
     {
-        appendRich(QObject::tr("Can not open file %1").arg(ff));
-        return;
+        ff = file;  // .fwb file path already includes wdir
+        wfile = new ifstream(ff.latin1());
+        if ( ! *wfile)
+        {
+            addToLog(QObject::tr("Can not open file %1").arg(ff));
+            delete wfile;
+            return;
+        }
     }
 
     /* need to convert strings of the config file from Utf-8 to
@@ -901,9 +955,9 @@ void instDialog::initiateCopy(const QString &file)
      * before sending to the installer process
      */
     string s0;
-    while ( !config_file->eof() )
+    while ( !wfile->eof() )
     {
-        getline( *config_file, s0);
+        getline( *wfile, s0);
 //        QString s = QString::fromUtf8(s0.c_str());
 //        QString s = s0.c_str();
 //        s.stripWhiteSpace();
@@ -912,9 +966,9 @@ void instDialog::initiateCopy(const QString &file)
 
         allConfig.push_back(s0);
     }
-    config_file->close();
-    delete config_file;
-    config_file=NULL;
+    wfile->close();
+    delete wfile;
+    wfile=NULL;
 
 //    allConfig.push_back("\004");
 
@@ -972,9 +1026,8 @@ void instDialog::initiateCopy(const QString &file)
 
     args.push_back(cmd);
         
-    appendRich( tr("\nCopying %1 -> %2:%3\n")
+    addToLog( tr("\nCopying %1 -> %2:%3\n")
                 .arg(cnf.wdir+"/"+file).arg(cnf.maddr).arg(cnf.fwdir) );
-    appendRich( "\n");
 
     if (cnf.verbose) displayCommand(args);
 
@@ -982,7 +1035,8 @@ void instDialog::initiateCopy(const QString &file)
 
     QTextCodec::setCodecForCStrings(QTextCodec::codecForName("latin1"));
 
-    SSHSession *s= new SSHUnx(cnf.fwobj->getName().c_str(),
+    SSHSession *s= new SSHUnx(this,
+                              cnf.fwobj->getName().c_str(),
                               args,
                               cnf.pwd,
                               "",
@@ -1006,9 +1060,9 @@ void instDialog::displayCommand(const QStringList &args)
         }
     }
     QString s=a1.join(" ");
-    appendRich( tr("Running command '%1'\n\n").arg(s) );
-    appendRich("\n");
+    addToLog( tr("Running command '%1'\n\n").arg(s) );
 }
+
 void instDialog::finishInstall(bool success)
 {
     if (fwbdebug) qDebug("instDialog::finishInstall");
@@ -1021,8 +1075,8 @@ void instDialog::finishInstall(bool success)
     }
     else
     {
-        opListMapping[*opListIterator]->setText(1,tr("Failure"));
-        processedFirewalls[*opListIterator].second=tr("Failure");
+        opListMapping[*opListIterator]->setText(1,tr("Error"));
+        processedFirewalls[*opListIterator].second=tr("Error");
     }
     
     opListIterator++;
@@ -1034,6 +1088,7 @@ void instDialog::finishInstall(bool success)
     }
     setNextEnabled( page(1), true);
 }
+
 void instDialog::continueRun()
 {
     if (fwbdebug) qDebug("instDialog::continueRun");
@@ -1043,7 +1098,7 @@ void instDialog::continueRun()
         if (session->getErrorStatus())
         {
             if (fwbdebug) qDebug("session error");
-            appendRich( tr("ERROR: Terminating install sequence\n") );
+            addToLog( tr("Fatal error, terminating install sequence\n") );
             finishInstall(false);
             //setFinishEnabled( page(1), true );
             return;
@@ -1056,7 +1111,7 @@ void instDialog::continueRun()
     if (activationCommandDone)
     {
         if (fwbdebug) qDebug("activationCommandDone");
-        appendRich( tr("Done\n") );
+        addToLog( tr("Done\n") );
         finishInstall();
         return;
     }
@@ -1115,14 +1170,15 @@ void instDialog::continueRun()
             QString cmd = getActivationCmd();
             args.push_back( cmd );
 
-            appendRich( tr("Activating new policy\n") );
-            appendRich( "\n");
+            addToLog( tr("Activating new policy\n") );
+            addToLog( "\n");
 
             if (cnf.verbose) displayCommand(args);
 
             activationCommandDone=true;
 
-            runSSH( new SSHUnx(cnf.fwobj->getName().c_str(),
+            runSSH( new SSHUnx(this,
+                               cnf.fwobj->getName().c_str(),
                                args,
                                cnf.pwd,
                                "",
@@ -1145,7 +1201,10 @@ void instDialog::runSSH(SSHSession *s)
             this,SLOT(append(const QString&)));
 
     connect(session,SIGNAL(sessionFinished_sign()),
-            this,SLOT(processExited()));
+            this,SLOT(installerFinished()));
+
+    connect(session,SIGNAL(sessionFatalError_sign()),
+            this,SLOT(installerError()));
 
     connect(session,SIGNAL(updateProgressBar_sign(int,bool)),
             this,SLOT(updateProgressBar(int,bool)));
@@ -1176,7 +1235,10 @@ void instDialog::reject()
                    this,SLOT(append(const QString&)));
 
         disconnect(session,SIGNAL(sessionFinished_sign()),
-                   this,SLOT(processExited()));
+                   this,SLOT(installerFinished()));
+
+        disconnect(session,SIGNAL(sessionFatalError_sign()),
+                   this,SLOT(installerError()));
 
         disconnect(session,SIGNAL(updateProgressBar_sign(int,bool)),
                    this,SLOT(updateProgressBar(int,bool)));
@@ -1207,6 +1269,7 @@ void instDialog::testRunRequested()
 #if 0
 #endif
 }
+
 void instDialog::saveLog()
 {
     QString dir;
@@ -1214,7 +1277,20 @@ void instDialog::saveLog()
     dir=st->getWDir();
     if (dir.isEmpty())  dir=st->getOpenFileDir();
     if (dir.isEmpty())  dir="~";
-    
+
+    /*
+     * We use QTextEdit::append to add lines to the log buffer, each
+     append creates a new paragraph so QTextEdit::text returns only
+     contents of the last paragraph. Need to reassemble the whole text
+     adding text from each paragraph separately.
+     */
+    QString logText;
+    for (int i=0; i<currentLog->paragraphs(); ++i)
+    {
+        logText += currentLog->text(i);
+        logText += "\n";
+    }
+
     QString s = QFileDialog::getSaveFileName(
                     dir,
                     "Text file (*.txt)",
@@ -1223,16 +1299,21 @@ void instDialog::saveLog()
                     "Choose a file" );
     
     
+    if (fwbdebug)
+        qDebug( "Saving log to file %s", s.ascii() );
+
     if (!s.isEmpty())
     {
         if (s.findRev(".txt",-1,false)==-1)
         {
             s+=".txt";
         }
+
         QFile f(s);
-        if (f.open(IO_WriteOnly))
+        if (f.open(IO_WriteOnly ))
         {
-            QTextStream (&f) << currentLog->text();
+            QTextStream str( &f );
+            str << logText;
             f.close();
         }
     }    
@@ -1241,7 +1322,7 @@ void instDialog::saveLog()
 void instDialog::findFirewalls()
 {
     firewalls.clear();
-    om->findFirewallsInCurrentLib(firewalls);
+    om->findAllFirewalls(firewalls);
     firewalls.sort(FWObjectNameCmpPredicate());
     saveMCLogButton->setEnabled(true);
 }
@@ -1256,7 +1337,7 @@ bool instDialog::runCompile(Firewall *fw)
     }
 
    
-    appendRich( QString("\n") + 
+    addToLog( QString("\n") + 
                 QObject::tr("Compiling rule sets for firewall: %1").arg(
                     fw->getName().c_str()
                 )
@@ -1296,10 +1377,10 @@ bool instDialog::testFirewall(Firewall *fw)
     {
         QMessageBox::critical(this, "Firewall Builder",
    tr("Policy installer uses Secure Shell to communicate with the firewall.\n"
-      "Please configure directory path to the secure file copy and secure \n"
-      "shell utilities installed on your machine using Preferences dialog"),
+      "Please configure directory path to the secure shell utility \n"
+      "installed on your machine using Preferences dialog"),
                               tr("&Continue") );
-        appendRich("Please configure directory path to the secure file copy and secure \n shell utilities installed on your machine using Preferences dialog");
+        addToLog("Please configure directory path to the secure \n shell utility installed on your machine using Preferences dialog");
         return false;
     }
 
@@ -1315,7 +1396,7 @@ bool instDialog::testFirewall(Firewall *fw)
     if ( !QFile::exists(fwfname) )
     {
 /* need to recompile */
-        appendRich(tr("Firewall isn't compiled."));
+        addToLog(tr("Firewall isn't compiled."));
         if (fwbdebug) qDebug("Firewall isn't compiled.");
         return false;
     }
@@ -1500,112 +1581,181 @@ Can't compile firewall policy."),
     return true;
 }
 
-void instDialog::readFromStdout()
+/*
+ * Adds one line of text to the log
+ *
+ */
+void instDialog::addToLog(const QString &line)
 {
+    if (fwbdebug)
+        qDebug("instDialog::addToLog");
+
     if (currentLog)
     {
-        currentLog->moveCursor( QTextEdit::MoveEnd , false );
-        QString buf = proc.readStdout();
-        QStringList bufLines = QStringList::split("\n",buf);
-        for (QStringList::Iterator i=bufLines.begin(); i!=bufLines.end(); ++i)
+        if (line.isEmpty()) return;
+
+//        currentLog->moveCursor( QTextEdit::MoveEnd , false );
+
+        QStringList words=QStringList::split(" ",line);
+
+#if 0  
+// although it is nice to be able to print errors in red, this
+// will break because of localization
+        QColor oc=currentLog->color();
+        if (words.first().find("Error")>=0) currentLog->setColor(Qt::red);
+        if (words.first().find("Abnormal")>=0) currentLog->setColor(Qt::red);
+        if (words.first().find("Warning")>=0) currentLog->setColor(Qt::blue);
+#endif        
+//        currentLog->insert( line );
+        currentLog->append( line );
+        
+        if (words.first().find("rule")>=0)
         {
-            QString line= *i;
-            QStringList words=QStringList::split(" ",line);
-        
-            QColor oc=currentLog->color();
-            if (words.first().find("Error")>=0) currentLog->setColor(Qt::red);
-            if (words.first().find("Warning")>=0) currentLog->setColor(Qt::blue);
-        
-            currentLog->insert( line );
-            currentLog->insert( "\n" );
-            currentLog->setColor(Qt::black);
-        
-            if (words.first().find("rule")>=0)
+            currentProgressBar->setProgress(++processedRules);
+        } else
+        {
+            if (words.first().find("processing")>=0)
             {
-                currentProgressBar->setProgress(++processedRules);
+                currentProgressBar->reset();
+                totalRules=words[1].toInt();
+                currentProgressBar->setTotalSteps(totalRules);
+                processedRules=0;
             } else
             {
-                if (words.first().find("processing")>=0)
+                if (words.first().find("Compiling")>=0)
                 {
+                    currentLabel->setText(line.stripWhiteSpace());
                     currentProgressBar->reset();
-                    totalRules=words[1].toInt();
-                    currentProgressBar->setTotalSteps(totalRules);
-                    processedRules=0;
                 } else
                 {
-                    if (words.first().find("Compiling")>=0)
+                    if (line.find("Compiled successfully")>=0)
                     {
                         currentLabel->setText(line.stripWhiteSpace());
-                        currentProgressBar->reset();
-                    } else
-                    {
-                        if (line.find("Compiled successfully")>=0)
-                        {
-                            currentLabel->setText(line.stripWhiteSpace());
-                            currentProgressBar->setProgress(currentProgressBar->totalSteps());
-                        }
+                        currentProgressBar->setProgress(currentProgressBar->totalSteps());
                     }
                 }
             }
-            QApplication::eventLoop()->processEvents(QEventLoop::ExcludeUserInput,1);
         }
+        QApplication::eventLoop()->processEvents(QEventLoop::ExcludeUserInput,1);
+
+        if (fwbdebug)
+            qDebug( QString("instDialog::addToLog Current log buffer contents"
+                            ": %1 lines, %2 paragraphs, '%3'").
+                    arg(currentLog->lines()).
+                    arg(currentLog->paragraphs()).
+                    arg(currentLog->text()) );
+        
     }
 }
 
+void instDialog::readFromStdout()
+{
+    QString buf = proc.readStdout();
+    bool endsWithLF = buf.endsWith("\n");
+    QString lastLine = "";
+
+    // split on LF
+    QStringList bufLines = QStringList::split("\n",buf,TRUE);
+
+    if (fwbdebug)
+    {
+        qDebug("buf=%s",buf.ascii());
+        qDebug("endsWithLF=%d",endsWithLF);
+        qDebug("bufLines.size()=%d",bufLines.size());
+    }
+
+    // if buf ends with a LF character, the last element in the list is
+    // an empty string
+    if (endsWithLF && bufLines.last().isEmpty())  bufLines.pop_back();
+
+    // if buf does not end with LF, last element in the list is
+    // incomplete line of text
+    if (!endsWithLF)
+    {
+        lastLine = bufLines.last();
+        bufLines.pop_back();
+    }
+
+    // elements that are left in the list  are all complete lines of text
+    for (QStringList::Iterator i=bufLines.begin(); i!=bufLines.end(); ++i)
+    {
+        addToLog( pendingLogLine + *i + "\n");
+        pendingLogLine = "";
+    }
+
+    pendingLogLine += lastLine;
+}
+
+void instDialog::installerError()
+{
+    if (fwbdebug) qDebug("instDialog::installerError");
+
+    addToLog( tr("Error: Terminating install sequence\n") );
+    finishInstall(false);
+
+    resetInstallSSHSession();
+    //setFinishEnabled( page(1), true );
+
+    if (session) delete session;
+    session=NULL;
+}
+
+void instDialog::installerFinished()
+{
+    if( fwbdebug) qDebug("instDialog::installerFinished");
+        
+    if (session->getErrorStatus())
+    {
+        installerError();
+    }
+
+    if (session) delete session;
+    session=NULL;
+
+    QTimer::singleShot( 0, this, SLOT(continueRun()) );
+}
+
 void instDialog::processExited()
 {
-    if( fwbdebug)
-    { qDebug("instDialog::processExited");}
+    if( fwbdebug) qDebug("instDialog::processExited  ");
     
     int res=proc.exitStatus();
-    
-    if (!compileFlag)
+
+    if (!compileFlag && customScriptFlag)
     {
-        if( fwbdebug)
-        { qDebug("installation");}
-        
-        if (customScriptFlag)
+        if( fwbdebug) qDebug("Custom script installer");
+            
+        if(res==0)
         {
-            if( fwbdebug) qDebug("Custom script installer");
+            om->updateLastInstalledTimestamp(*opListIterator);
+            processedFirewalls[*opListIterator].second="Success";
+            opListMapping[*opListIterator]->setText(1,tr("Success"));
+        }   
+        else
+        {
+            processedFirewalls[*opListIterator].second="Error";
+            opListMapping[*opListIterator]->setText(1,tr("Error"));
+        }
+        currentProgressBar->setProgress(currentProgressBar->totalSteps () ); 
+        qApp->processEvents();
             
-            
-            if(res==0)
-            {
-                om->updateLastInstalledTimestamp(*opListIterator);
-                processedFirewalls[*opListIterator].second="success";
-                opListMapping[*opListIterator]->setText(1,tr("Success"));
-            }   
-            else
-            {
-                processedFirewalls[*opListIterator].second="faild";
-                opListMapping[*opListIterator]->setText(1,tr("Faild"));
-            }
-            currentProgressBar->setProgress(currentProgressBar->totalSteps () ); 
-            qApp->processEvents();
-            
-            opListIterator++;
+        opListIterator++;
         
-            if (opListIterator!=opList.end() && batchInstall->isChecked() && !stopProcessFlag)
-            {
-                installSelected();
-            }
-            else
-            {
-                currentSaveButton->setEnabled(true);
-                setNextEnabled(page(1), true);
-                saveMCLogButton->setEnabled(true);
-            }
+        if (opListIterator!=opList.end() && batchInstall->isChecked() && !stopProcessFlag)
+        {
+            installSelected();
         }
         else
         {
-            if( fwbdebug) qDebug(QString("Buildin installer res= %1").arg(res));
-            QTimer::singleShot( 0, this, SLOT(continueRun()) );
+            currentSaveButton->setEnabled(true);
+            setNextEnabled(page(1), true);
+            saveMCLogButton->setEnabled(true);
         }
         return;
     }
+
         
-   
-    if(res==0)   
+    if(res==0 && proc.normalExit())   
     {
         om->updateLastCompiledTimestamp(*opListIterator);
         processedFirewalls[*opListIterator].first=tr("Success");
@@ -1613,16 +1763,23 @@ void instDialog::processExited()
     }
     else
     {
+        addToLog( tr("Abnormal program termination") +"\n");
         // WARNING:
-        // If compilation of the firewall failed we should not to
+        // If compilation of the firewall failed we should not
         // install it
         //
         installMapping[*opListIterator]->setChecked(false);
         processedFirewalls[*opListIterator].second=tr("Skipped");
-        processedFirewalls[*opListIterator].first=tr("Failure");
-        opListMapping[*opListIterator]->setText(1,tr("Failure"));
+        processedFirewalls[*opListIterator].first=tr("Error");
+        opListMapping[*opListIterator]->setText(1,tr("Error"));
     }
-    
+
+    if (fwbdebug)
+        qDebug( QString(" Current log buffer contents: %1 lines, '%2'").
+                arg(currentLog->lines()).arg(currentLog->text()) );
+
+
+
     Firewall *f;
     QListViewItem* item;
     
@@ -1670,6 +1827,7 @@ void instDialog::processExited()
     currentSaveButton->setEnabled(true);
 
 }
+
 void instDialog::selectAllFirewalls()
 {
     if (fwbdebug) qDebug("instDialog::selectAllFirewalls");
@@ -1744,7 +1902,7 @@ void instDialog::compileSelected()
     
     currentLog = procLogDisplay;
     currentSaveButton = saveMCLogButton;
-    currentSaveButton->setEnabled(false);
+    currentSaveButton->setEnabled(true);
     currentStopButton = controlMCButton;
     currentProgressBar = compProgress;
     currentFirewallsBar = compFirewallProgress;
@@ -1816,11 +1974,13 @@ void instDialog::stopCompile()
     QTimer::singleShot( 1000, &proc, SLOT( kill() ) );
     
 }
+
 void instDialog::stopInstall()
 {
     currentStopButton->setEnabled(false);
     stopProcessFlag=true;
 }
+
 void instDialog::fillLastList()
 {
     lastListView->clear();
@@ -1847,6 +2007,7 @@ void instDialog::fillLastList()
     
     lastListView->setSorting(0);
 }
+
 bool instDialog::runInstall(Firewall * fw)
 {
     if (fwbdebug) qDebug("instDialog::runInstall");
@@ -1857,11 +2018,11 @@ bool instDialog::runInstall(Firewall * fw)
         proc.clearArguments();
 
         proc.setArguments( args );
-        appendRich( proc.arguments().join(" ") );
-        appendRich("\n");
+        addToLog( proc.arguments().join(" ") );
+        addToLog("\n");
         if ( !proc.start() )
         {
-            appendRich( tr("Error: Failed to start program") );
+            addToLog( tr("Error: Failed to start program") );
             return false;
         }
         
@@ -1873,6 +2034,7 @@ bool instDialog::runInstall(Firewall * fw)
     }
     return true;
 }
+
 void instDialog::fillInstallOpList()
 {
     if (fwbdebug) qDebug("instDialog::fillInstallOpList");
@@ -1902,6 +2064,9 @@ void instDialog::fillInstallOpList()
 }
 void instDialog::initInstall()
 {
+    if (fwbdebug)
+        qDebug("instDialog::initInstall()");
+
     currentFirewallsBar =compFirewallProgress;
     currentFirewallsBar->reset();
     currentFirewallsBar->setTotalSteps(opList.size());
@@ -1922,6 +2087,7 @@ void instDialog::initInstall()
     progress=0;
     stopProcessFlag=false;
 }
+
 void instDialog::installSelected()
 {
     if (fwbdebug) qDebug("instDialog::installSelected");
@@ -1939,14 +2105,14 @@ void instDialog::installSelected()
     while ( opListIterator!=opList.end())
     {
     
-        currentSaveButton->setEnabled(false);
+        currentSaveButton->setEnabled(true);
         currentProgressBar->reset();
         currentProgressBar->setTotalSteps(100);
         
         currentLabel->setText(QString::fromUtf8((*opListIterator)->getName().c_str()));
         compileFlag=false;
         
-        doOptionsPage();
+        resetInstallSSHSession();
         currentFirewallsBar->setProgress(++progress);
 
         appendRich("<hr>");
@@ -2007,7 +2173,7 @@ void instDialog::installSelected()
                         }
                         else
                         {
-                            processedFirewalls[*opListIterator].second="cancelled";
+                            processedFirewalls[*opListIterator].second="Cancelled";
                             opListMapping[*opListIterator]->setText(1,tr("Failure"));
                             opListIterator++;
                             setNextEnabled(page(1),true);
@@ -2162,8 +2328,10 @@ void instDialog::showSelected()
     showSelectedFlag=!showSelectedFlag;
     
 }
+
 void instDialog::tableValueChanged(int row, int col)
 {
+    if (fwbdebug) qDebug("instDialog::tableValueChanged");
     QCheckTableItem *item;
     Firewall *f;
     
@@ -2194,6 +2362,7 @@ void instDialog::tableValueChanged(int row, int col)
     
     setNextEnabled(page(0), isTableHasChecked());
 }
+
 bool instDialog::isTableHasChecked()
 {
     QCheckTableItem *item;
@@ -2215,6 +2384,7 @@ bool instDialog::isTableHasChecked()
     }
     return res;
 }
+
 void instDialog::analyseInstallQueue(bool &fPix, bool &fCustInst)
 {
     if (fwbdebug) qDebug("instDialog::analyseInstallQueue");
@@ -2250,10 +2420,12 @@ void instDialog::analyseInstallQueue(bool &fPix, bool &fCustInst)
         if (fPix && !fCustInst) return;// nothing can change if we continue loop
     }
 }
+
 void instDialog::clearReqFirewalls()
 {
     reqFirewalls.clear();
 }
+
 void instDialog::addReqFirewall(Firewall *f)
 {
     reqFirewalls.insert(f);
