@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: main.cpp,v 1.111 2006/07/20 23:11:10 vkurland Exp $
+  $Id: main.cpp,v 1.112 2006/09/08 05:43:59 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -40,6 +40,7 @@
 #include <qapplication.h>
 #include <qtextcodec.h>
 #include <qtooltip.h>
+#include <qtimer.h>
 
 /*
 #ifdef _WIN32
@@ -125,7 +126,7 @@ QTextEdit         *oi         = NULL;
 FWBSettings       *st         = NULL;
 findDialog        *fd         = NULL;
 int                fwbdebug   = 0;
-int                safemode   = 0;
+bool               safemode   = false;
 bool               registered = false;
 bool               gui_experiment1 = false;
 
@@ -329,7 +330,6 @@ void usage()
 }
 
 
-
 int main( int argc, char ** argv )
 {
     bool        ssh_wrapper=false;
@@ -339,7 +339,7 @@ int main( int argc, char ** argv )
     filename="";
     objid="";
     fwbdebug=0;
-    safemode=0;
+    safemode=false;
 
     bool desktopaware = true;
 
@@ -561,7 +561,7 @@ int main( int argc, char ** argv )
             break;
 
         case 's':
-            safemode++;
+            safemode = true;
             break;
             
         case 'g':
@@ -671,91 +671,19 @@ int main( int argc, char ** argv )
         fd  = new findDialog(mw);
         fd->hide();
 
-        
-        if (fwbdebug) qDebug("loading data file ...");
-
-        if (safemode)  mw->load(NULL);
-        else
-        {
-            if (filename!="")
-            {
-                try
-                {
-                    RCS *rcs=new RCS(filename);
-                    rcs->co();
-                    mw->load(NULL,rcs);
-                } catch (FWException &ex)
-                {
-                    qDebug("Exception: %s",ex.toString().c_str());
-                    mw->load(NULL);
-                }
-            } else
-            {
-                int sa = st->getStartupAction();
-                switch (sa) 
-                {
-                case 0:   // load standard objects
-                {
-                    mw->load(NULL); // load standard objects
-                    break;
-                }
-                case 1:   // load last edited file
-                {
-                    filename=st->getLastEdited();
-                    if (filename!="")
-                    {
-                        try
-                        {
-                            RCS *rcs=new RCS(filename);
-                            rcs->co();
-                            mw->load(NULL,rcs);
-                        } catch (FWException &ex)
-                        {
-                            qDebug("Exception: %s",ex.toString().c_str());
-                            mw->load(NULL);
-                        }
-                    } else
-                        mw->load(NULL);
-                    break;
-                }
-                default:  //  load standard objects too
-                {
-                    mw->load(NULL); // load standard objects
-                }
-                }
-            }
-        }
-
-        mw->showFirewalls();
-
-        if ( st->getStartupAction()==1 && safemode==0 )
-        {
-            QString id = st->getStr("UI/visibleFirewall");
-            FWObject *o=NULL;
-            if ( !id.isEmpty() ) o=mw->db()->getById(id.latin1(),true);
-            if (o)
-            {
-                if (fwbdebug)
-                    qDebug("open firewall %s",o->getName().c_str());
-                mw->showFirewall(o);
-            }
-
-            id = st->getStr("UI/visibleObject");
-            o=NULL;
-            if ( !id.isEmpty() ) o=mw->db()->getById(id.latin1(),true);
-            if (o)
-            {
-                if (fwbdebug)
-                    qDebug("open object %s",o->getName().c_str());
-                om->openObject(o);
-            }
-        }
+        mw->setSafeMode(safemode);
+        mw->setStartupFileName(filename);
 
         QToolTip::setWakeUpDelay( st->getTooltipDelay()*1000 );
 
         mw->show();
 
         app->connect( app, SIGNAL( lastWindowClosed() ), app, SLOT( quit() ) );
+
+// setup single shot timer to call loadEverything()
+
+        QTimer::singleShot( 500, mw, SLOT(startupLoad()) );
+
         app->exec();
 
         oe->hide();
