@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: newFirewallDialog.cpp,v 1.27 2006/06/16 04:33:13 vkurland Exp $
+  $Id: newFirewallDialog.cpp,v 1.30 2006/09/11 00:41:09 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -94,7 +94,8 @@ newFirewallDialog::newFirewallDialog() : newFirewallDialog_q()
 
     iface_sl_list->setItemMargin( 1 );
     iface_sl_list->setAllColumnsShowFocus( true );
-
+    QToolTip::add(iface_dyn,wordWrap(tr("Check option 'dynamic address' for the interface that gets its IP address dynamically via DHCP or PPP protocol.") ,80 ));
+    QToolTip::add(iface_unnum,wordWrap(tr("Check option 'Unnumbered interface' for the interface that does not have an IP address. Examples of interfaces of this kind are those used to terminate PPPoE or VPN tunnels.") ,80 ));
     obj_name->setFocus();
 }
 
@@ -368,7 +369,7 @@ void newFirewallDialog::selected(const QString &title)
 #endif
 
         list<FWObject*> fl;
-        findFirewalls(tlib, fl);
+        findFirewalls(tlib, fl, false);
 
         QString icn_filename = QString( Resources::global_res->getObjResourceStr(fl.front(), "icon-tree").c_str() );
 
@@ -735,7 +736,8 @@ void newFirewallDialog::accept()
                 oa->setAddress( addr.latin1()    );
                 oa->setNetmask( netmask.latin1() );
             }
-
+            // updateObjName has a side effect: it causes redraw of the ruleset
+            // views in the main window
             om->updateObjName(oi,"","",false);
 
             itm=itm->itemBelow();
