@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: newHostDialog.cpp,v 1.10 2006/06/16 04:33:13 vkurland Exp $
+  $Id: newHostDialog.cpp,v 1.11 2006/08/09 08:21:07 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -343,7 +343,7 @@ void newHostDialog::selected(const QString &title)
 #endif
 
         list<FWObject*> fl;
-        findHosts(tlib, fl);
+        findHosts(tlib, fl, false);
 
         QString icn_filename = QString( Resources::global_res->getObjResourceStr(fl.front(), "icon-tree").c_str() );
 
