@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: platforms.cpp,v 1.32 2006/07/22 15:26:37 vkurland Exp $
+  $Id: platforms.cpp,v 1.33 2006/09/17 18:58:12 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -222,7 +222,12 @@ bool isDefaultPolicyRuleOptions(FWOptions *opt)
 	    res= ( opt->getStr("log_prefix").empty()       &&  
 		   ! opt->getBool("stateless")             &&
                    opt->getInt("pf_rule_max_state")<=0     &&
-                   ! opt->getBool("pf_source_tracking") );
+                   ! opt->getBool("pf_source_tracking")    &&
+                   opt->getInt("pf_max_src_conn")<=0       &&
+                   opt->getInt("pf_max_src_conn_rate_num")<=0       &&
+                   opt->getInt("pf_max_src_conn_rate_seconds")<=0
+
+            );
 	}
 
 	if (platform=="ipf") 
