@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: NATCompiler_PrintRule.cpp,v 1.4 2004/12/29 07:56:23 vkurland Exp $
+  $Id: NATCompiler_PrintRule.cpp,v 1.13 2006/06/05 04:53:21 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -24,6 +24,7 @@
 */
 
 #include "NATCompiler_ipt.h"
+#include "OSConfigurator_linux24.h"
 
 #include "fwbuilder/AddressRange.h"
 #include "fwbuilder/RuleElement.h"
@@ -32,11 +33,14 @@
 #include "fwbuilder/TCPService.h"
 #include "fwbuilder/UDPService.h"
 #include "fwbuilder/CustomService.h"
+#include "fwbuilder/TagService.h"
 #include "fwbuilder/Policy.h"
 #include "fwbuilder/Interface.h"
 #include "fwbuilder/IPv4.h"
 #include "fwbuilder/Firewall.h"
 #include "fwbuilder/Resources.h"
+#include "fwbuilder/DNSName.h"
+#include "fwbuilder/AddressTable.h"
 
 #include "combinedAddress.h"
 
@@ -69,13 +73,13 @@ using namespace std;
  */
 string NATCompiler_ipt::PrintRule::_createChain(const string &chain)
 {
-    string res;
+    ostringstream  res;
     if ( ! chains[chain] )
     {
-	res = "$IPTABLES -t nat -N " + chain + "\n";
+	res << "$IPTABLES -t nat -N " << chain << endl;
 	chains[chain]=true;
     }
-    return res;
+    return res.str();
 }
 
 string NATCompiler_ipt::PrintRule::_startRuleLine()
@@ -173,7 +177,7 @@ string NATCompiler_ipt::PrintRule::_printChainDirectionAndInterface(NATRule *rul
 string NATCompiler_ipt::PrintRule::_printProtocol(Service *srv)
 {
     std::ostringstream  ostr;
-    if ( ! srv->isAny() && ! CustomService::isA(srv) ) 
+    if (!srv->isAny() && !CustomService::isA(srv) && !TagService::isA(srv))
     {
         string pn=srv->getProtocolName();
         if (pn=="ip") pn="all";
@@ -315,7 +319,7 @@ string NATCompiler_ipt::PrintRule::_printSrcService(RuleElementOSrv  *rel)
  */
     FWObject *o=rel->front();
     if (o && FWReference::cast(o)!=NULL)
-        o=compiler->getCachedObj( FWReference::cast(o)->getPointerId() );
+        o=FWReference::cast(o)->getPointer();
 
     Service *srv= Service::cast(o);
 
@@ -331,7 +335,7 @@ string NATCompiler_ipt::PrintRule::_printSrcService(RuleElementOSrv  *rel)
 	bool  first=true;
 	for (FWObject::iterator i=rel->begin(); i!=rel->end(); i++) {
 	    FWObject *o= *i;
-	    if (FWReference::cast(o)!=NULL) o=compiler->getCachedObj(o->getStr("ref"));
+	    if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
 
 	    Service *s=Service::cast( o );
 	    assert(s);
@@ -344,7 +348,7 @@ string NATCompiler_ipt::PrintRule::_printSrcService(RuleElementOSrv  *rel)
 	if ( !str.empty() )
         {
             string v = compiler->fw->getStr("version");
-            if (v.empty() || v=="ge_1.2.6" || v=="1.2.9")
+            if (v.empty() || v=="ge_1.2.6" || v=="1.2.9" || v=="1.3.0")
                 ostr << "--sports ";
             else
                 ostr << "--source-port ";
@@ -360,7 +364,7 @@ string NATCompiler_ipt::PrintRule::_printDstService(RuleElementOSrv  *rel)
 
     FWObject *o=rel->front();
     if (o && FWReference::cast(o)!=NULL)
-        o=compiler->getCachedObj( FWReference::cast(o)->getPointerId() );
+        o=FWReference::cast(o)->getPointer();
 
     Service *srv= Service::cast(o);
 
@@ -385,6 +389,11 @@ string NATCompiler_ipt::PrintRule::_printDstService(RuleElementOSrv  *rel)
         {
 	    ostr << CustomService::cast(srv)->getCodeForPlatform( compiler->myPlatformName() ) << " ";
 	}
+        if (TagService::isA(srv))
+        {
+	    ostr << "--mark "
+                 << TagService::cast(srv)->getCode() << " ";
+        }
 
     } else {
 /* use multiport */
@@ -394,7 +403,7 @@ string NATCompiler_ipt::PrintRule::_printDstService(RuleElementOSrv  *rel)
 	for (FWObject::iterator i=rel->begin(); i!=rel->end(); i++)
         {
 	    FWObject *o= *i;
-	    if (FWReference::cast(o)!=NULL) o=compiler->getCachedObj(o->getStr("ref"));
+	    if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
 
 	    Service *s=Service::cast( o );
 	    assert(s);
@@ -408,7 +417,7 @@ string NATCompiler_ipt::PrintRule::_printDstService(RuleElementOSrv  *rel)
 	if ( !str.empty() )
         {
             string v = compiler->fw->getStr("version");
-            if (v.empty() || v=="ge_1.2.6" || v=="1.2.9")
+            if (v.empty() || v=="ge_1.2.6" || v=="1.2.9" || v=="1.3.0")
                 ostr << "--dports ";
             else
                 ostr << "--destination-port ";
@@ -423,10 +432,30 @@ string NATCompiler_ipt::PrintRule::_printAddr(Address  *o,bool print_mask,bool p
     NATCompiler_ipt *ipt_comp=dynamic_cast<NATCompiler_ipt*>(compiler);
     std::ostringstream  ostr;
 
+    MultiAddressRunTime *atrt = MultiAddressRunTime::cast(o);
+    if (atrt!=NULL)
+    {
+        if (atrt->getSubstitutionTypeName()==AddressTable::TYPENAME)
+        {
+            ostr << "$" << ipt_comp->getAddressTableVarName(atrt) << " ";
+            return ostr.str();
+        }
+
+        if (atrt->getSubstitutionTypeName()==DNSName::TYPENAME)
+        {
+            return atrt->getSourceName();
+        }
+        // at this time we only support two types of MultiAddress
+        // objects: AddressTable and DNSName. Both should be converted
+        // to MultiAddressRunTime at this point. If we get some other
+        // kind of MultiAddressRunTime object, we do not know what to do
+        // with it so we stop.
+        assert(atrt==NULL);
+    }
+
     IPAddress addr=o->getAddress();
     Netmask   mask=o->getNetmask();
     Interface *iface;
-
     if ( (iface=Interface::cast(o))!=NULL ) 
     {
         if (iface->isDyn() && iface->getBool("use_var_address"))
@@ -493,11 +522,10 @@ bool NATCompiler_ipt::PrintRule::processNext()
     compiler->output << _printRuleLabel(rule);
 
     string s;
-    std::ostringstream  command_line;
-
-    command_line << _createChain(rule->getStr("ipt_chain"));
-    command_line << _createChain(rule->getStr("ipt_target"));
+    std::ostringstream  cmdout;
 
+    compiler->output << _createChain(rule->getStr("ipt_chain"));
+    compiler->output << _createChain(rule->getStr("ipt_target"));
 
 
 //    RuleElementOSrc *osrcrel=rule->getOSrc();
@@ -515,14 +543,14 @@ bool NATCompiler_ipt::PrintRule::processNext()
 //	Interface::cast( rule->getRoot()->getById(rule->getInterfaceId() ,true) );
 
 
-    command_line << _startRuleLine();
+    cmdout << _startRuleLine();
 
-    command_line << _printChainDirectionAndInterface(rule);
+    cmdout << _printChainDirectionAndInterface(rule);
 
     if (! osrv->isAny() )
-	command_line << _printProtocol(osrv);
+	cmdout << _printProtocol(osrv);
 
-    command_line << _printMultiport(rule);
+    cmdout << _printMultiport(rule);
 
     if (!osrc->isAny()) 
     {
@@ -536,7 +564,7 @@ bool NATCompiler_ipt::PrintRule::processNext()
 
         if ( ! physaddress.empty())
         {
-	    command_line << " -m mac --mac-source "
+	    cmdout << " -m mac --mac-source "
                              << physaddress;
         }
 /*
@@ -548,39 +576,37 @@ bool NATCompiler_ipt::PrintRule::processNext()
             ;
         } else
         {
-            command_line << " -s ";
-            command_line << _printAddr(osrc);
+            cmdout << " -s ";
+            cmdout << _printAddr(osrc);
         }
 
-
-
-
-//	command_line << " -s ";
-//	command_line << _printAddr(osrc);
+//	cmdout << " -s ";
+//	cmdout << _printAddr(osrc);
     }
 
     if (!osrv->isAny()) {
-	command_line << " ";
-	command_line << _printSrcService(osrvrel);
+	cmdout << " ";
+	cmdout << _printSrcService(osrvrel);
     }
 
     if (!odst->isAny()) {
-	command_line << " -d ";
-	command_line << _printAddr(odst);
+	cmdout << " -d ";
+	cmdout << _printAddr(odst);
     }
 
-    command_line << " ";
-    command_line << _printDstService(osrvrel);
+    cmdout << " ";
+    cmdout << _printDstService(osrvrel);
 
-    command_line << "-j " << rule->getStr("ipt_target") << " ";
+    cmdout << "-j " << rule->getStr("ipt_target") << " ";
 
     switch (rule->getRuleType()) {
     case NATRule::SNAT:  
-	if (rule->getStr("ipt_target")=="SNAT") {
-	    command_line << "--to-source ";
-	    command_line << _printAddr(tsrc,false,true);
+	if (rule->getStr("ipt_target")=="SNAT")
+        {
+	    cmdout << "--to-source ";
+	    cmdout << _printAddr(tsrc,false,true);
 	    string ports=_printSNATPorts(tsrv);
-	    if (!ports.empty()) command_line << ":" << ports;
+	    if (!ports.empty()) cmdout << ":" << ports;
 	}
 	break;
 /*
@@ -590,82 +616,57 @@ bool NATCompiler_ipt::PrintRule::processNext()
  *  to do the right thing.
  */
     case NATRule::DNAT:  
-	if (rule->getStr("ipt_target")=="DNAT") {
-	    command_line << "--to-destination ";
-            if (!tdst->isAny()) command_line << _printAddr(tdst,false,true);
+	if (rule->getStr("ipt_target")=="DNAT")
+        {
+	    cmdout << "--to-destination ";
+            if (!tdst->isAny()) cmdout << _printAddr(tdst,false,true);
 	    string ports=_printDNATPorts(tsrv);
-	    if (!ports.empty()) command_line << ":" << ports;
+	    if (!ports.empty()) cmdout << ":" << ports;
 	}
 	break;
 
     case NATRule::SNetnat:
-        command_line << "--to ";
-        command_line << _printAddr(tsrc,true,false);
+	if (rule->getStr("ipt_target")=="NETMAP")
+        {
+            cmdout << "--to ";
+            cmdout << _printAddr(tsrc,true,false);
+	}
         break;
-
+  
     case NATRule::DNetnat:
-        command_line << "--to ";
-        command_line << _printAddr(tdst,true,false);
+	if (rule->getStr("ipt_target")=="NETMAP")
+        {
+            cmdout << "--to ";
+            cmdout << _printAddr(tdst,true,false);
+	}
         break;
-
+  
     case NATRule::Redirect: 
-	if (rule->getStr("ipt_target")=="REDIRECT") {
+	if (rule->getStr("ipt_target")=="REDIRECT")
+        {
 	    string ports=_printDNATPorts(tsrv);
-	    if (!ports.empty()) command_line << "--to-ports " << ports;
+	    if (!ports.empty()) cmdout << "--to-ports " << ports;
 	}
 	break;
     default:	break;
     }
-    command_line << " ";
+    cmdout << " ";
 
-    command_line << _endRuleLine();
+    cmdout << _endRuleLine();
 
-//    command_line  << endl;
-
-
-/* if anywhere in command_line we used variable holding an address of
- * dynamic interface (named $i_something) then we need to add
- * this command with a check for the value of this variable. We execute
- * iptables command only if the value is a non-empty string.
- */
-
-    string::size_type p1=command_line.str().find("$i_");
-    string  iface_name;
-    string  iface_var;
-    if ( p1!=string::npos )
-    {
-        string::size_type p2=command_line.str().find(" ",p1);
-        string::size_type p3=command_line.str().find("_",p1) +1;
-        iface_name=command_line.str().substr(p3,p2-p3);
-        iface_var= command_line.str().substr(p1,p2-p1);
-
-/* if interface name ends with '*', this is a wildcard interface. */
-        string::size_type p4;
-        if ((p4=iface_name.find("*"))!=string::npos)
-        {
-            string cmdline=command_line.str();
-            string iface_family_name=iface_name.substr(0,p4);
-            compiler->output << "getinterfaces " << iface_family_name << " | while read I; do" << endl;
-            compiler->output << "  ivar=`getInterfaceVarName $I`" << endl;
-            compiler->output << "  getaddr $I $ivar" << endl;
-            compiler->output << "  cmd=\"$\"$ivar"   << endl;
-            compiler->output << "  eval \"addr=$cmd\""          << endl;
-            cmdline.replace(p1,p2-p1,"$addr");
-            compiler->output << "  test -n \"$addr\" && " << cmdline;
-            compiler->output << "done" << endl;
-
-            return true;
-        } else
-            compiler->output << "test -n \"" << iface_var << "\" && ";
-    }
-
-
-    compiler->output << command_line.str();
+//    cmdout  << endl;
 
+    compiler->output 
+        << dynamic_cast<OSConfigurator_linux24*>(compiler->osconfigurator)->printRunTimeWrappers( rule, cmdout.str());
 
     return true;
 }
 
+string NATCompiler_ipt::PrintRule::_declareTable()
+{
+    return "";
+}
+
 string NATCompiler_ipt::PrintRule::_flushAndSetDefaultPolicy()
 {
     return "";
