@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyCompiler_ipt.h,v 1.22 2006/06/15 05:06:08 vkurland Exp $
+  $Id: PolicyCompiler_ipt.h,v 1.23 2006/08/19 21:15:20 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -224,6 +224,11 @@ namespace fwcompiler {
         };
 
 
+	/**
+	 * set chain if Tag rule should go into PREROUTING
+	 */
+        DECLARE_POLICY_RULE_PROCESSOR(setChainIfTagInPrerouting);
+
 	/**
 	 * split rule if action is Tag and connmark option is activated
 	 */
