@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: ipt.cpp,v 1.55 2006/06/05 04:53:22 vkurland Exp $
+  $Id: ipt.cpp,v 1.59 2006/09/15 05:48:22 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -248,10 +248,12 @@ int main(int argc, char * const *argv)
         objdb->merge(ndb, NULL);
         delete ndb;
         objdb->setFileName(filename);
+        objdb->reIndex();
 
-//	objdb->load(filename,  &upgrade_predicate, librespath);
 	if (verbose) cout << _(" done\n");
 
+        //objdb->dump(true,true);
+
         FWObject *slib = objdb->findInIndex("syslib000");
         if ( slib->isReadOnly()) slib->setReadOnly(false);
 
@@ -752,7 +754,7 @@ _("Dynamic interface %s should not have an IP address object attached to it. Thi
         return 0;
 
     } catch(const FWException &ex)  {
-	cerr << ex.toString() << endl;
+	cerr << "Error: " << ex.toString() << endl;
         return 1;
 #if __GNUC__ >= 3
 /* need to check version because std::ios::failure does not seem to be
