@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyCompiler_ipf_writers.cpp,v 1.4 2005/03/30 07:33:18 vkurland Exp $
+  $Id: PolicyCompiler_ipf_writers.cpp,v 1.8 2006/06/17 21:53:54 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -36,6 +36,7 @@
 #include "fwbuilder/IPv4.h"
 #include "fwbuilder/FWOptions.h"
 #include "fwbuilder/Firewall.h"
+#include "fwbuilder/DNSName.h"
 
 #include <iostream>
 
@@ -113,8 +114,7 @@ string PolicyCompiler_ipf::PrintRule::_printPort(int rs,int re,bool neg)
 void PolicyCompiler_ipf::PrintRule::_printDstService(RuleElementSrv  *rel)
 {
     FWObject *o=rel->front();
-    if (o && FWReference::cast(o)!=NULL)
-        o=compiler->getCachedObj( FWReference::cast(o)->getPointerId() );
+    if (o && FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
 
     Service *srv= Service::cast(o);
 
@@ -138,10 +138,22 @@ void PolicyCompiler_ipf::PrintRule::_printAction(PolicyRule *rule)
     Service *srv=compiler->getFirstSrv(rule);    assert(srv);
 
     switch (rule->getAction()) {
-    case PolicyRule::Skip:        compiler->output << "skip " << rule->getInt("no_to_skip") << " "; break;
-    case PolicyRule::Accept:      compiler->output << "pass  "; break;
-    case PolicyRule::Accounting:  compiler->output << "count "; break;
-    case PolicyRule::Deny:        compiler->output << "block "; break;
+    case PolicyRule::Skip:
+        compiler->output << "skip " << rule->getInt("no_to_skip") << " ";
+        break;
+
+    case PolicyRule::Accept:
+        compiler->output << "pass  ";
+        break;
+
+    case PolicyRule::Accounting:
+        compiler->output << "count ";
+        break;
+
+    case PolicyRule::Deny:
+        compiler->output << "block ";
+        break;
+
     case PolicyRule::Reject: 
         if (rule->getDirection()==PolicyRule::Inbound)
         {
@@ -176,6 +188,11 @@ void PolicyCompiler_ipf::PrintRule::_printAction(PolicyRule *rule)
         } else
             compiler->output << "block ";
 	break;
+
+    case PolicyRule::Custom:
+        compiler->output << ruleopt->getStr("custom_str") << " ";
+        break;
+
     default:   compiler->output << rule->getActionAsString() << " ";
     }
 }
@@ -223,6 +240,22 @@ void PolicyCompiler_ipf::PrintRule::_printAddr(Address  *o,bool neg)
 {
     FWOptions* options=compiler->fw->getOptionsObject();
 
+    MultiAddressRunTime *atrt = MultiAddressRunTime::cast(o);
+    if (atrt!=NULL)
+    {
+        if (atrt->getSubstitutionTypeName()==DNSName::TYPENAME)
+        {
+            compiler->output <<  atrt->getSourceName() << " ";
+            return;
+        }
+        // at this time we only support two types of MultiAddress
+        // objects: AddressTable and DNSName. Both should be converted
+        // to MultiAddressRunTime at this point. If we get some other
+        // kind of MultiAddressRunTime object, we do not know what to do
+        // with it so we stop.
+        assert(atrt==NULL);
+    }
+
     IPAddress addr=o->getAddress();
     Netmask   mask=o->getNetmask();
 
@@ -322,6 +355,9 @@ bool PolicyCompiler_ipf::PrintRule::processNext()
 
     if ( rule->getBool("quick") ) compiler->output << "quick ";
     _printInterface(rule);
+
+    _printRouteOptions(rule);
+
     _printProtocol(srv);
 
     compiler->output << " from ";
