@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyCompiler_ipfw.cpp,v 1.8 2006/05/17 02:30:18 vkurland Exp $
+  $Id: PolicyCompiler_ipfw.cpp,v 1.11 2006/09/09 05:05:02 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -36,6 +36,7 @@
 #include "fwbuilder/Policy.h"
 #include "fwbuilder/Interface.h"
 #include "fwbuilder/Firewall.h"
+#include "fwbuilder/AddressTable.h"
 
 #include <iostream>
 
@@ -51,17 +52,17 @@ int PolicyCompiler_ipfw::prolog()
 {
     int n= PolicyCompiler_pf::prolog();
 
-    anytcp=TCPService::cast(dbcopy->create(TCPService::TYPENAME,false) );
+    anytcp=TCPService::cast(dbcopy->create(TCPService::TYPENAME) );
     anytcp->setId(ANY_TCP_OBJ_ID);
     dbcopy->add(anytcp,false);
     cacheObj(anytcp); // to keep cache consistent
 
-    anyudp=UDPService::cast(dbcopy->create(UDPService::TYPENAME,false) );
+    anyudp=UDPService::cast(dbcopy->create(UDPService::TYPENAME) );
     anyudp->setId(ANY_UDP_OBJ_ID);
     dbcopy->add(anyudp,false);
     cacheObj(anyudp); // to keep cache consistent
 
-    anyicmp=ICMPService::cast(dbcopy->create(ICMPService::TYPENAME,false) );
+    anyicmp=ICMPService::cast(dbcopy->create(ICMPService::TYPENAME) );
     anyicmp->setId(ANY_ICMP_OBJ_ID);
     dbcopy->add(anyicmp,false);
     cacheObj(anyicmp); // to keep cache consistent
@@ -99,7 +100,7 @@ bool PolicyCompiler_ipfw::expandAnyService::processNext()
     if (srv->isAny() && ! ruleopt->getBool("stateless") && rule->getAction()==PolicyRule::Accept) 
     {
 	PolicyRule *r= PolicyRule::cast(
-	    compiler->dbcopy->create(PolicyRule::TYPENAME,false) );
+	    compiler->dbcopy->create(PolicyRule::TYPENAME) );
 	compiler->temp_ruleset->add(r);
 	r->duplicate(rule);
 	RuleElementSrv *nsrv=r->getSrv();
@@ -108,7 +109,7 @@ bool PolicyCompiler_ipfw::expandAnyService::processNext()
 	tmp_queue.push_back(r);
 
 	r= PolicyRule::cast(
-	    compiler->dbcopy->create(PolicyRule::TYPENAME,false) );
+	    compiler->dbcopy->create(PolicyRule::TYPENAME) );
 	compiler->temp_ruleset->add(r);
 	r->duplicate(rule);
 	nsrv=r->getSrv();
@@ -117,7 +118,7 @@ bool PolicyCompiler_ipfw::expandAnyService::processNext()
 	tmp_queue.push_back(r);
 
 	r= PolicyRule::cast(
-	    compiler->dbcopy->create(PolicyRule::TYPENAME,false) );
+	    compiler->dbcopy->create(PolicyRule::TYPENAME) );
 	compiler->temp_ruleset->add(r);
 	r->duplicate(rule);
 	nsrv=r->getSrv();
@@ -126,7 +127,7 @@ bool PolicyCompiler_ipfw::expandAnyService::processNext()
 	tmp_queue.push_back(r);
 
 	r= PolicyRule::cast(
-	    compiler->dbcopy->create(PolicyRule::TYPENAME,true) );
+	    compiler->dbcopy->create(PolicyRule::TYPENAME) );
 	compiler->temp_ruleset->add(r);
 	r->duplicate(rule);
 	FWOptions *ruleopt =r->getOptionsObject();
@@ -149,7 +150,7 @@ bool PolicyCompiler_ipfw::doSrcNegation::processNext()
         RuleElementSrc *nsrc;
 	PolicyRule     *r;
 
-	r= PolicyRule::cast(compiler->dbcopy->create(PolicyRule::TYPENAME,false) );
+	r= PolicyRule::cast(compiler->dbcopy->create(PolicyRule::TYPENAME) );
 	compiler->temp_ruleset->add(r);
 	r->duplicate(rule);
 	r->setAction(PolicyRule::Continue);
@@ -160,7 +161,7 @@ bool PolicyCompiler_ipfw::doSrcNegation::processNext()
         r->setBool("skip_check_for_duplicates",true);
 	tmp_queue.push_back(r);
 
-	r= PolicyRule::cast(compiler->dbcopy->create(PolicyRule::TYPENAME,false) );
+	r= PolicyRule::cast(compiler->dbcopy->create(PolicyRule::TYPENAME) );
 	compiler->temp_ruleset->add(r);
 	r->duplicate(rule);
         nsrc=r->getSrc();
@@ -187,7 +188,7 @@ bool PolicyCompiler_ipfw::doDstNegation::processNext()
         RuleElementDst *ndst;
 	PolicyRule     *r;
 
-	r= PolicyRule::cast(compiler->dbcopy->create(PolicyRule::TYPENAME,false) );
+	r= PolicyRule::cast(compiler->dbcopy->create(PolicyRule::TYPENAME) );
 	compiler->temp_ruleset->add(r);
 	r->duplicate(rule);
 	r->setAction(PolicyRule::Continue);
@@ -198,7 +199,7 @@ bool PolicyCompiler_ipfw::doDstNegation::processNext()
         r->setBool("skip_check_for_duplicates",true);
 	tmp_queue.push_back(r);
 
-	r= PolicyRule::cast(compiler->dbcopy->create(PolicyRule::TYPENAME,false) );
+	r= PolicyRule::cast(compiler->dbcopy->create(PolicyRule::TYPENAME) );
 	compiler->temp_ruleset->add(r);
 	r->duplicate(rule);
         ndst=r->getDst();
@@ -267,7 +268,7 @@ bool PolicyCompiler_ipfw::separatePortRanges::processNext()
                 if (sawServiceWithPortRange)
                 {
                     PolicyRule *r= PolicyRule::cast(
-                        compiler->dbcopy->create(PolicyRule::TYPENAME,false) );
+                        compiler->dbcopy->create(PolicyRule::TYPENAME) );
                     compiler->temp_ruleset->add(r);
                     r->duplicate(rule);
                     RuleElementSrv *nsrv=r->getSrv();
@@ -477,6 +478,27 @@ bool PolicyCompiler_ipfw::eliminateDuplicateRules::processNext()
     return true;
 }
 
+/*
+ * this processor is the same as in PolicyCompiler_ipf
+ */
+bool PolicyCompiler_ipfw::processMultiAddressObjectsInRE::processNext()
+{
+    PolicyRule *rule=getNext(); if (rule==NULL) return false;
+    RuleElement *re=RuleElement::cast( rule->getFirstByType(re_type) );
+
+    for (FWObject::iterator i=re->begin(); i!=re->end(); i++)
+    {
+        FWObject *o= *i;
+        if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
+        MultiAddressRunTime *atrt = MultiAddressRunTime::cast(o);
+        if (atrt!=NULL && atrt->getSubstitutionTypeName()==AddressTable::TYPENAME)
+            compiler->abort("Run-time AddressTable objects are not supported. Rule " + rule->getLabel());
+    }
+
+    tmp_queue.push_back(rule);
+    return true;
+}
+
 
 void PolicyCompiler_ipfw::compile()
 {
@@ -506,6 +528,10 @@ void PolicyCompiler_ipfw::compile()
             add( new eliminateDuplicatesInSRC("eliminate duplicates in SRC"));
             add( new eliminateDuplicatesInDST("eliminate duplicates in DST"));
             add( new eliminateDuplicatesInSRV("eliminate duplicates in SRV"));
+
+            add( new swapMultiAddressObjectsInSrc(" swap MultiAddress -> MultiAddressRunTime in Src") );
+            add( new swapMultiAddressObjectsInDst(" swap MultiAddress -> MultiAddressRunTime in Dst") );
+
             add( new ExpandMultipleAddressesInSRC(
                      "expand objects with multiple addresses in SRC"));
             add( new ExpandMultipleAddressesInDST(
@@ -547,6 +573,8 @@ void PolicyCompiler_ipfw::compile()
         add( new swapMultiAddressObjectsInSrc(" swap MultiAddress -> MultiAddressRunTime in Src") );
         add( new swapMultiAddressObjectsInDst(" swap MultiAddress -> MultiAddressRunTime in Dst") );
 
+        add( new processMultiAddressObjectsInSrc("process MultiAddress objects in Src") );
+        add( new processMultiAddressObjectsInDst("process MultiAddress objects in Dst") );
 
         add( new splitIfFirewallInSrc("split rule if firewall is in Src"));
         add( new splitIfFirewallInDst("split rule if firewall is in Dst"));
