@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyCompiler_ipfw.h,v 1.4 2005/04/04 07:00:54 vkurland Exp $
+  $Id: PolicyCompiler_ipfw.h,v 1.5 2006/08/31 16:14:29 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -165,6 +165,33 @@ namespace fwcompiler {
         };
         friend class fwcompiler::PolicyCompiler_ipfw::eliminateDuplicateRules;
 
+        /**
+         * Placeholders for MultiAddressRunTime objects which are not
+         * supported for ipfw
+         */
+        class processMultiAddressObjectsInRE : public PolicyRuleProcessor
+        {
+            std::string re_type;
+            public:
+            processMultiAddressObjectsInRE(const std::string &name,
+                      const std::string &t) : PolicyRuleProcessor(name) { re_type=t; }
+            virtual bool processNext();
+        };
+
+
+        class processMultiAddressObjectsInSrc : public processMultiAddressObjectsInRE
+        {
+            public:
+            processMultiAddressObjectsInSrc(const std::string &n) :
+                processMultiAddressObjectsInRE(n,libfwbuilder::RuleElementSrc::TYPENAME) {}
+        };
+
+        class processMultiAddressObjectsInDst : public processMultiAddressObjectsInRE
+        {
+            public:
+            processMultiAddressObjectsInDst(const std::string &n) :
+                processMultiAddressObjectsInRE(n,libfwbuilder::RuleElementDst::TYPENAME) {}
+        };
 
 	/**
 	 *   prints single policy rule, assuming all groups have been
