@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyCompiler_ipfw_writers.cpp,v 1.7 2005/05/30 22:09:19 vkurland Exp $
+  $Id: PolicyCompiler_ipfw_writers.cpp,v 1.11 2006/05/17 02:30:18 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -23,6 +23,8 @@
 
 */
 
+#include "definitions.h"
+
 #include "PolicyCompiler_ipfw.h"
 #include "OSData.h"
 
@@ -39,6 +41,7 @@
 #include "fwbuilder/FWOptions.h"
 #include "fwbuilder/Firewall.h"
 #include "fwbuilder/Resources.h"
+#include "fwbuilder/DNSName.h"
 
 #include <iostream>
 
@@ -173,10 +176,22 @@ void PolicyCompiler_ipfw::PrintRule::_printAction(PolicyRule *rule)
     Service *srv=compiler->getFirstSrv(rule);    assert(srv);
 
     switch (rule->getAction()) {
-    case PolicyRule::Skip:        compiler->output << "skipto " << rule->getInt("skip_to") << " "; break;
-    case PolicyRule::Accounting:  compiler->output << "count  "; break;
-    case PolicyRule::Accept:      compiler->output << "permit "; break;
-    case PolicyRule::Deny:        compiler->output << "drop   "; break;
+    case PolicyRule::Skip:
+        compiler->output << "skipto " << rule->getInt("skip_to") << " ";
+        break;
+
+    case PolicyRule::Accounting:
+        compiler->output << "count  ";
+        break;
+
+    case PolicyRule::Accept:
+        compiler->output << "permit ";
+        break;
+
+    case PolicyRule::Deny:
+        compiler->output << "drop   ";
+        break;
+
     case PolicyRule::Reject: 
         if (TCPService::isA(srv)) compiler->output << "reset  ";
         else 
@@ -203,6 +218,33 @@ void PolicyCompiler_ipfw::PrintRule::_printAction(PolicyRule *rule)
             compiler->output << "unreach " << code;
         }
 	break;
+
+    case PolicyRule::Classify:
+    {
+        int portNum = ruleopt->getInt("ipfw_pipe_queue_num");
+        switch (ruleopt->getInt("ipfw_classify_method"))
+        {
+        case DUMMYNETPIPE:
+            compiler->output << "pipe "   << portNum << " ";
+            break;
+        case DUMMYNETQUEUE:
+            compiler->output << "queue "  << portNum << " ";
+            break;
+        default:
+            compiler->output << "divert " << portNum << " ";
+            break;
+        }
+    }
+    break;
+
+    case PolicyRule::Pipe:
+        compiler->output << "divert " << ruleopt->getInt("ipfw_pipe_port_num") << " ";
+        break;
+
+    case PolicyRule::Custom:
+        compiler->output << ruleopt->getStr("custom_str") << " ";
+        break;
+
     default:   compiler->output << rule->getActionAsString() << " ";
     }
 }
@@ -220,6 +262,22 @@ void PolicyCompiler_ipfw::PrintRule::_printAddr(Address  *o,bool neg)
         return;
     }
 
+    MultiAddressRunTime *atrt = MultiAddressRunTime::cast(o);
+    if (atrt!=NULL)
+    {
+        if (atrt->getSubstitutionTypeName()==DNSName::TYPENAME)
+        {
+            compiler->output <<  atrt->getSourceName() << " ";
+            return;
+        }
+        // at this time we only support two types of MultiAddress
+        // objects: AddressTable and DNSName. Both should be converted
+        // to MultiAddressRunTime at this point. If we get some other
+        // kind of MultiAddressRunTime object, we do not know what to do
+        // with it so we stop.
+        assert(atrt==NULL);
+    }
+
     IPAddress addr=o->getAddress();
     Netmask   mask=o->getNetmask();
 
@@ -275,8 +333,7 @@ void PolicyCompiler_ipfw::PrintRule::_printSrcService(RuleElementSrv  *rel)
  * find the object. I'd rather use a cached copy in the compiler
  */
     FWObject *o=rel->front();
-    if (o && FWReference::cast(o)!=NULL)
-        o=compiler->getCachedObj( FWReference::cast(o)->getPointerId() );
+    if (o && FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
 
     Service *s1= Service::cast(o);
 
@@ -287,7 +344,7 @@ void PolicyCompiler_ipfw::PrintRule::_printSrcService(RuleElementSrv  *rel)
     for (list<FWObject*>::iterator i1=rel->begin(); i1!=rel->end(); ++i1) 
     {
         FWObject *o   = *i1;
-        if (FWReference::cast(o)!=NULL) o=compiler->getCachedObj(o->getStr("ref"));
+        if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
         Service *srv=Service::cast(o);
 
         if (tcpudp)
@@ -319,8 +376,7 @@ string PolicyCompiler_ipfw::PrintRule::_printSrcService(Service *srv,bool neg)
 void PolicyCompiler_ipfw::PrintRule::_printDstService(RuleElementSrv  *rel)
 {
     FWObject *o=rel->front();
-    if (o && FWReference::cast(o)!=NULL)
-        o=compiler->getCachedObj( FWReference::cast(o)->getPointerId() );
+    if (o && FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
 
     Service *s1= Service::cast(o);
 
@@ -335,7 +391,7 @@ void PolicyCompiler_ipfw::PrintRule::_printDstService(RuleElementSrv  *rel)
     for (list<FWObject*>::iterator i1=rel->begin(); i1!=rel->end(); ++i1) 
     {
         FWObject *o   = *i1;
-        if (FWReference::cast(o)!=NULL) o=compiler->getCachedObj(o->getStr("ref"));
+        if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
         Service *srv=Service::cast(o);
 
         if (tcpudp || custom)
