@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyCompiler_pf_writers.cpp,v 1.19 2006/06/17 21:53:54 vkurland Exp $
+  $Id: PolicyCompiler_pf_writers.cpp,v 1.20 2006/09/17 18:58:13 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -707,8 +707,12 @@ bool PolicyCompiler_pf::PrintRule::processNext()
 	    compiler->output << "keep state ";
 
         int nopt=0;
-        if (ruleopt->getInt("pf_rule_max_state")>0) nopt++;
-        if (ruleopt->getBool("pf_source_tracking")) nopt+=2;
+        if (ruleopt->getInt("pf_rule_max_state")>0)        nopt++;
+        if (ruleopt->getBool("pf_source_tracking"))        nopt+=2;
+        if (ruleopt->getInt("pf_max_src_conn")>0)          nopt++;
+        if (ruleopt->getStr("pf_max_src_conn_overload_table")!="") nopt++;
+        if (ruleopt->getInt("pf_max_src_conn_rate_num")>0) nopt++;
+        if (ruleopt->getStr("pf_max_src_conn_rate_overload_table")!="") nopt++;
 
         if (nopt)
         {
@@ -728,6 +732,37 @@ bool PolicyCompiler_pf::PrintRule::processNext()
                 compiler->output << " max-src-states "
                                  << ruleopt->getInt("pf_max_src_states");
             }
+
+            if (ruleopt->getInt("pf_max_src_conn")>0)
+            {
+                compiler->output << " max-src-conn "
+                                 << ruleopt->getInt("pf_max_src_conn");
+                if (ruleopt->getStr("pf_max_src_conn_overload_table")!="")
+                    compiler->output << ", overload <"
+                                     << ruleopt->getStr("pf_max_src_conn_overload_table") << ">";
+                if (ruleopt->getBool("pf_max_src_conn_flush"))
+                    compiler->output << " flush";
+                if (ruleopt->getBool("pf_max_src_conn_global"))
+                    compiler->output << " global";
+            }
+
+            if (ruleopt->getInt("pf_max_src_conn_rate_num")>0 && 
+                ruleopt->getInt("pf_max_src_conn_rate_seconds")>0)
+            {
+                compiler->output << " max-src-conn-rate "
+                                 << ruleopt->getInt("pf_max_src_conn_rate_num")
+                                 << "/"
+                                 << ruleopt->getInt("pf_max_src_conn_rate_seconds");
+
+                if (ruleopt->getStr("pf_max_src_conn_rate_overload_table")!="")
+                    compiler->output << ", overload <"
+                                     << ruleopt->getStr("pf_max_src_conn_rate_overload_table") << ">";
+                if (ruleopt->getBool("pf_max_src_conn_rate_flush"))
+                    compiler->output << " flush";
+                if (ruleopt->getBool("pf_max_src_conn_rate_global"))
+                    compiler->output << " global";
+            }
+
             if (nopt>1) compiler->output << " ) ";
         }
     }
