@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: TableFactory.cpp,v 1.9 2006/05/21 04:47:10 vkurland Exp $
+  $Id: TableFactory.cpp,v 1.10 2006/09/09 05:05:03 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -122,7 +122,7 @@ void TableFactory::createTablesForRE(RuleElement *re,Rule *rule)
         tblgrp = tables[tblID];
     } else
     {
-        tblgrp=ObjectGroup::cast(dbroot->create(ObjectGroup::TYPENAME,false));
+        tblgrp=ObjectGroup::cast(dbroot->create(ObjectGroup::TYPENAME));
 // TODO: can two rules yeild the same name for the group using this method?
         std::ostringstream tblname;
         if (!ruleSetName.empty()) tblname << ruleSetName << ":";
